/*****************
Propiedad intelectual del cdigo original de Computer Integrity (info@cintegrity.com)
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor original: T. Mercer-Hursh (thomas@cintegrity.com)
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Genera los diagramas del modelo de datos.
- ENGLISH -
Builds data model diagrams.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING PROGRESS.Lang.AppError.
USING com.nomadesoft.lib.*.
USING com.nomadesoft.ABLaUML.repository.*.
USING com.nomadesoft.ABLaUML.DDWorker.

/**
Se encarga de realizar los diagramas apropiados para el modelo de datos.
**/
CLASS com.nomadesoft.ABLaUML.DiagramDDWorker INHERITS DDWorker:
  DEFINE PUBLIC PROPERTY clgMainDiagram AS LOGICAL INITIAL TRUE NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY cchMainDiagramName AS CHARACTER INITIAL 'Vista de Datos' NO-UNDO
    GET.
    SET.

  DEFINE PROTECTED PROPERTY cinDBPackageID AS INTEGER NO-UNDO
    GET.
    PRIVATE SET.

  CONSTRUCTOR DiagramDDWorker(INPUT ipobUtils AS CLASS IUtils):
    SUPER(ipobUtils).
  END CONSTRUCTOR.

  
  METHOD PROTECTED OVERRIDE VOID UnknownStart():
    SUPER:UnknownStart().
    IF NOT clgMainDiagram THEN
      RETURN.

    DEFINE VARIABLE minDiagramID AS INTEGER     NO-UNDO.
    /* crear el diagrama general */
    minDiagramID = cobUtils:CreateDiagram(
        INPUT cinDataModelPackageId,
        cchMainDiagramName,
        'Package'
        ).
    IF minDiagramID EQ ? THEN
      UNDO, THROW NEW AppError('No se puede crear el diagrama',292).
    cinCurrentObjectID = minDiagramID.
  END METHOD.

  /* mtodos que los hijos sobrescribirn para hacer el trabajo real, invocados desde la implementacin de IXMLWorker */
  METHOD PROTECTED OVERRIDE VOID DatabaseStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE minDiagramID AS INTEGER     NO-UNDO.

    cinDBPackageId = cobUtils:FindPackage(
        input cobParser:AttributeValue('name'),
        input cinDataModelPackageID
        ). 
    IF cinDBPackageId EQ 0 THEN
      cinDBPackageID = ?.
    IF cinDBPackageID EQ ? THEN
      RETURN.
    IF clgMainDiagram THEN DO:
      minObjectID = cobUtils:FindObject (
        input "Package",
        input "nsDatabase",
        input cobParser:AttributeValue('name'),
        ?
        ).                  
      if  minObjectID GT 0 THEN
        cobUtils:AddDiagramObject(
          INPUT cinCurrentObjectID,
          INPUT minObjectID,
          '',
          FALSE
          ).
    END.
    /* crear el diagrama de la base de datos */
    minDiagramID = cobUtils:CreateDiagram(
        INPUT cinDBPackageId,
        cobParser:AttributeValue('name'),
        'Logical'
        ).

    IF minDiagramID EQ ? THEN
      UNDO, THROW NEW AppError('No se puede crear el diagrama',292).
    cinCurrentObjectID = minDiagramID.
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID TableStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.

    IF cinDBPackageID EQ ? THEN
      RETURN.

    minObjectID = cobUtils:FindObject (
        input "Class",
        input "nsTable",
        input cobParser:AttributeValue('name'),
        cinDBPackageID
        ).
    if  minObjectID EQ ? OR minObjectID EQ 0 THEN
      RETURN.

    cobUtils:AddDiagramObject(
      INPUT cinCurrentObjectID,
      INPUT minObjectID,
      '',
      FALSE
      ).
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID DatabaseEnd():
    IF cinDBPackageId NE ? THEN
      cinCurrentObjectID = ?.
    cinDBPackageId = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID UnknownEnd():
    cinCurrentObjectID = ?.
  END METHOD.
END CLASS.
