/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor original: T. Mercer-Hursh (thomas@cintegrity.com)
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Construye las relaciones entre los componentes y paquetes construidos
por ObjectsWorker.
- ENGLISH -
Establishes packages and components relations, previously builded by
ObjectsWorker.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING PROGRESS.Lang.AppError.
USING com.nomadesoft.ABLaUML.FUWorker.
USING com.nomadesoft.lib.*.
USING com.nomadesoft.ABLaUML.repository.*.

/**
Se encarga de crear todos los enlaces entre los objetos (los cuales ya deben existir).
**/
CLASS com.nomadesoft.ABLaUML.LinksWorker INHERITS FUWorker:

  DEFINE PROTECTED PROPERTY cchSkipTag AS CHARACTER INITIAL ? NO-UNDO
    GET.
    SET.

  DEFINE PRIVATE VARIABLE cchCurrentFU AS CHARACTER NO-UNDO.

  METHOD PROTECTED VOID AddLink(INPUT  ipinSourceID AS INTEGER,INPUT  ipinDestID AS INTEGER,INPUT  ipchStereotype AS CHARACTER):

    DEFINE VARIABLE iinConnectorID AS INTEGER     NO-UNDO.

    cobParser:doLog(string(this-object),SUBSTITUTE('Creando enlace entre &1 y &2, en unidad funcional &3',ipinSourceID, ipinDestID,cchCurrentFU),'DEBUG').
    iinConnectorID = cobUtils:CreateConnector (
      input "Dependency",
      input "",
      input "",
      input "",
      input "",
      input ipinSourceID,
      input ipinDestID,
      input ipchStereotype,
      cchCurrentFU NE '' AND cchCurrentFU NE ?
      ).
    if     iinConnectorID eq ?
      or iinConnectorID eq 0
    then
      undo, THROW NEW AppError("CreateConnector failed",221 ).
    cobUtils:SetConnectorAlias(iinConnectorID,cchCurrentFU).
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID FunctionalUnitStart():
    DEFINE VARIABLE mchProgramName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchTipo AS CHARACTER  INITIAL "nsInitialProgram" NO-UNDO.

    IF cchSkipTag NE ? THEN
      RETURN.
    mchProgramName = cobParser:AttributeValue("program").
    assign
      mchTipo = "nsInclude" WHEN (mchProgramName EQ ? OR mchProgramName EQ '')
      mchProgramName = cobParser:AttributeValue("include") WHEN (mchProgramName EQ ? OR mchProgramName EQ '')
      .
    minObjectID = cobUtils:FindObject (
          input "Component",
          input mchTipo,
          input mchProgramName,
          ?
          ).                  
    if     minObjectID eq ?
        or minObjectID eq 0
    then
      undo, throw new AppError("FindObject failed for = " + mchProgramName + " in cu",214 ).
    cinCurrentObjectID = minObjectID.
    cchCurrentFU = cobParser:AttributeValue('path').
    cinCurrentPackageID = cobUtils:FindPackage(
      cobParser:AttributeValue('path'),
      ?
      ).
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID IncludeStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.

    IF cchSkipTag NE ? THEN
      RETURN.
    minObjectID = cobUtils:FindObject (
          input "Component",
          input "nsInclude",
          input cobParser:AttributeValue("file"),
          cinCurrentPackageID
          ).                  
    if     minObjectID eq ?
        or minObjectID eq 0
    then
      undo, throw new AppError("FindObject failed for = " + cobParser:AttributeValue("file") + " in include",214 ).
    /* crear el link con quien contiene este archivo */
    AddLink(cinCurrentObjectID,minObjectID,'nsIncludeLink').
    cinCurrentObjectID = minObjectID.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID ProgramStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
  
    IF cchSkipTag NE ? THEN
      RETURN.
    minObjectID = cobUtils:FindObject (
          input "Component",
          input "nsProgram",
          input cobParser:AttributeValue("file"),
          cinCurrentPackageID
          ).                  
    if     minObjectID eq ?
        or minObjectID eq 0
    then
      undo, throw new AppError("FindObject failed for = " + cobParser:AttributeValue("file") + " in include",214 ).
    /* crear el link con quien contiene este archivo */
    AddLink(cinCurrentObjectID,minObjectID,'nsRunLink').
    cinCurrentObjectID = minObjectID.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID CallStart():
    DEFINE VARIABLE mchCalledName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.

    IF cchSkipTag NE ? THEN
      RETURN.
    mchCalledName = cobParser:AttributeValue('component').
    ASSIGN
      mchCalledName =  cobParser:AttributeValue("fu") WHEN (mchCalledName EQ ? OR mchCalledName EQ '')
      .
    IF mchCalledName MATCHES  cchClasifMatch THEN DO:
      minObjectID = cobUtils:FindObject(
        input "Component",
        input "nsClasifInclude",
        input mchCalledName,
        cinClasifComponentsPackageId
        ).
      IF minObjectID EQ ? OR minObjectID EQ 0 THEN DO:
        minObjectID = cobUtils:FindObject(
          input "Component",
          input "nsClasifProgram",
          input mchCalledName,
          cinClasifComponentsPackageId
          ).
      END.
    END.
    ELSE IF mchCalledName MATCHES cchLibMatch THEN DO:
      minObjectID = cobUtils:FindObject(
        input "Component",
        input "nsLibInclude",
        input mchCalledName,
        cinnsLibComponentsPackageId
        ).
      IF minObjectID EQ ? OR minObjectID EQ 0 THEN DO:
        minObjectID = cobUtils:FindObject(
          input "Component",
          input "nsLibProgram",
          input mchCalledName,
          cinnsLibComponentsPackageId
          ).
      END.
    END.
    ELSE DO:
      mchCalledName = cobParser:AttributeValue("fu").
      minObjectID = cobUtils:FindObject (
        input "Package",
        input "nsFunctionalUnit",
        input mchCalledName,
        ?
        ).                  
    END.
    /* es aceptable que falten unidades funcionales */
    if     minObjectID eq ?
        or minObjectID eq 0
    THEN DO:
      cobParser:doLog(string(this-object),"No se ha encontrado la unidad funcional " + mchCalledName,'ERROR').
      cchSkipTag = cobParser:CurrentElement().
    END.
    ELSE DO:
      /* crear el link con quien contiene este archivo */
      IF NUM-ENTRIES(mchCalledName,'.') GT 1 AND ENTRY(2,mchCalledName,'.') EQ 'i' THEN
        AddLink(cinCurrentObjectID,minObjectID,'nsIncludeLink').
      ELSE
        AddLink(cinCurrentObjectID,minObjectID,'nsRunLink').
      cinCurrentObjectID = minObjectID.
    END.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID TableRefStart():
    DEFINE VARIABLE mchTableName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchDBName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mlgCRUD AS LOGICAL  EXTENT 4   NO-UNDO.


    mchTableName = SplitDBTable(cobParser:AttributeValue("table"),OUTPUT mchDBName).
    minObjectID = cobUtils:FindPackage(
        mchDBName,
        cinDataModelPackageID
        ).
    minObjectID = cobUtils:FindObject (
        input "Class",
        input "nsTable",
        input mchTableName,
        minObjectID
        ).
    if     minObjectID eq ?
        or minObjectID eq 0
    THEN
      undo, throw new AppError("FindObject failed for " + mchTableName + " in tableref",242 ).

    ASSIGN
      mlgCRUD[1] = cobParser:AttributeValue("create") eq "true"
      mlgCRUD[2] = cobParser:AttributeValue("read") eq "true"
      mlgCRUD[3] = cobParser:AttributeValue("write") eq "true"
      mlgCRUD[4] = cobParser:AttributeValue("delete") eq "true"
      .
  
    minObjectID = cobUtils:SetDataLink(
              INPUT cinCurrentObjectID,
              input minObjectID,
              input mlgCRUD[1],
              input mlgCRUD[2],
              input mlgCRUD[3],
              input mlgCRUD[4],
              input FALSE,
              INPUT cchCurrentFU
              ).
      if     minObjectID eq ?
          or minObjectID eq 0
      then
        undo, throw new AppError("DataLink failed for " + mchTableName + " in tableref ",243 ).  
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID FunctionalUnitEnd():
    IF cchSkipTag EQ ? THEN DO:
      cinCurrentObjectID = ?.
      cinCurrentPackageID = ?.
    END.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID IncludeEnd():
    IF cchSkipTag EQ ? THEN
      cinCurrentObjectID = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID ProgramEnd():
    IF cchSkipTag EQ ? THEN
      cinCurrentObjectID = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID CallEnd():
    IF cchSkipTag EQ cobParser:CurrentElement() THEN
      cchSkipTag = ?.
    ELSE IF cchSkipTag EQ ? THEN
      cinCurrentObjectID = ?.
  END METHOD.

  CONSTRUCTOR LinksWorker(INPUT ipobUtils AS CLASS IUtils):
    SUPER(ipobUtils).
  END CONSTRUCTOR.
END CLASS.
