/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase que coordina el procesamiento de un xml y su uso por uno o ms
constructores. Dado un xml y una lista de constructores, se encarga
de que los mismos procesen el xml en el orden en que estn en la
lista, para ello utiliza un XMLParser, EAUtils y stMapper.
- ENGLISH -
Workers coordination class. Given an xml and a list of workers, it
instantiates workers and makes them process the xml file. It uses
an XMLParser, EAUtils and stMapper.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING com.nomadesoft.ABLaUML.*.
USING com.nomadesoft.ABLaUML.repository.*.
USING com.nomadesoft.lib.*.
USING Progress.Lang.AppError.

CLASS com.nomadesoft.ABLaUML.UMLBuilder:
  DEFINE PRIVATE VARIABLE cobParser AS IXMLParser NO-UNDO.
  DEFINE PRIVATE VARIABLE cobUtils AS IUtils    NO-UNDO.
  DEFINE PRIVATE VARIABLE cobMapper AS StMapper NO-UNDO.

  DEFINE PUBLIC PROPERTY cchStereotypeMap AS CHARACTER NO-UNDO
    GET.
    SET(INPUT ipchNewMap AS CHARACTER):
      cobMapper:cchMap = ipchNewMap.
      cchStereotypeMap = cobMapper:cchMap.
      /* reasigno el mapper, para que se actualice el estado interno */
      CAST(cobUtils,EAUtils):cobStereotypeMapper = cobMapper.
    END SET.

  CONSTRUCTOR PUBLIC UMLBuilder(INPUT ipobLogger AS CLASS ILogger,INPUT iplgIncremental AS LOGICAL):
    cobMapper = NEW stMapper().
    cobUtils = NEW EAUtils(iplgIncremental).
    CAST(cobUtils,EAUtils):cobStereotypeMapper = cobMapper.
    IF NOT iplgIncremental THEN
      CAST(cobUtils,EAUtils):CleanIt().
    cobParser = NEW XMLParser().
    CAST(cobParser,XMLParser):setLogger(CAST(cobUtils,EAUtils):setLogger(ipobLogger)).
  END CONSTRUCTOR.

  /** destructor **/
  DESTRUCTOR UMLBuilder():
    DELETE OBJECT cobParser.  
    DELETE OBJECT cobUtils.
  END DESTRUCTOR.

  /* recibe el nombre del archivo XML a procesar y un listado de XMLWorkers que instanciar */
  METHOD PUBLIC VOID Build(INPUT ipchBoM AS CHARACTER, INPUT ipchWorkers AS CHARACTER):
    DEFINE VARIABLE mchDiskFile AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchEntry AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mobWorker AS IXMLWorker NO-UNDO.

    mchDiskFile = search( ipchBoM ).
    if mchDiskFile eq ? then
      undo, THROW NEW AppError("Cannot find " + ipchBoM + " - aborting" ).
    
    DO minIndex = 1 TO NUM-ENTRIES(ipchWorkers) ON ERROR UNDO, THROW:
      mchEntry = ENTRY(minIndex, ipchWorkers).
      mobWorker = DYNAMIC-NEW(mchEntry)(cobUtils).
      cobParser:setWorker(mobWorker).
      cobParser:parse(mchDiskFile).
      FINALLY:
        DELETE OBJECT mobWorker.
      END FINALLY.
    END.
  END METHOD.
END CLASS.
