/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase que interpreta un xml e invoca los mtodos adecuados de un
constructor asociado.
- ENGLISH -
Class for xml parsing, that invokes associated worker methods.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING com.nomadesoft.lib.*.

/** esta clase implementa el procesamiento del XML, invocando una clase delegada (a travs de una interfaz) para realizar el trabajo real.
Es bsicamente un encapsulamiento de SAX. **/
CLASS com.nomadesoft.ABLaUML.XMLParser IMPLEMENTS IXMLParser, ILoggeable:
  /** propiedades **/
  /* indica si actualmente se est leyendo XML con SAX o no. Se lo utiliza para validar las invocaciones de los mtodos de interpretacin
  del XML */
  DEFINE PUBLIC PROPERTY clgInSAX AS LOGICAL NO-UNDO
    GET.
    PRIVATE SET.

  /* cchCurrentTag es siempre vlido, tanto al inicio como al finalizar el marcador, sin embargo los atributos son vlidos hasta
  la apertura de un nuevo marcador, lo que significa que al cierre es posible que los atributos correspondan a un marcador hijo
  del actual*/
  DEFINE PUBLIC PROPERTY cchCurrentTag AS CHARACTER NO-UNDO
    GET.
    PRIVATE SET.
  
  DEFINE PUBLIC PROPERTY cchEncoding AS CHARACTER INITIAL 'UTF-8' NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY cchCurrentContent AS CHARACTER NO-UNDO
    GET.
    PRIVATE SET.
  DEFINE PUBLIC PROPERTY cchAttributes AS CHARACTER NO-UNDO
    GET.
    PRIVATE SET.
  DEFINE PUBLIC PROPERTY cchAttributesValues AS CHARACTER NO-UNDO
    GET.
    PRIVATE SET.
  
  DEFINE PROTECTED PROPERTY cchTagHistory AS CHARACTER NO-UNDO
    GET.
    PRIVATE SET.

  DEFINE PUBLIC PROPERTY cobWorker AS CLASS IXMLWorker NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY cobLogger AS CLASS ILogger NO-UNDO
    GET.
    SET.
    
  /** atributos **/

  /* conserva el handle del .p que realiza la lectura del XML utilizando SAX, y que invoca los mtodos de este objeto */
  DEFINE PRIVATE VARIABLE chnSAXp AS HANDLE      NO-UNDO.

  /** constructores **/
  CONSTRUCTOR PUBLIC XMLParser():
    RUN com/nomadesoft/lib/SAX.p PERSISTENT SET chnSAXp (THIS-OBJECT).
  END CONSTRUCTOR.

  /** destructor **/
  DESTRUCTOR XMLParser():
    DELETE PROCEDURE chnSAXp NO-ERROR.  
  END DESTRUCTOR.


  /** mtodos **/
  /* convierte la salida en el cdigo de caracteres apropiado */
  METHOD PROTECTED CHARACTER EncodeString(INPUT ipchString AS CHARACTER):
    RETURN CODEPAGE-CONVERT(ipchString,cchEncoding).
  END METHOD.

  METHOD PUBLIC VOID Parse(INPUT iphnStream AS HANDLE):
    clgInSAX = TRUE.
    RUN Parse IN chnSAXp (?,iphnStream).
    FINALLY:
      clgInSAX = FALSE.
    END FINALLY.
  END METHOD.
  METHOD PUBLIC VOID Parse(INPUT ipchFile AS CHARACTER):
    clgInSAX = TRUE.
    RUN Parse IN chnSAXp (ipchFile,?).
    FINALLY:
      clgInSAX = FALSE.
    END FINALLY.
  END METHOD.

  METHOD PUBLIC CHARACTER BuildAttribute(INPUT ipinIndex AS INTEGER):
    DEFINE VARIABLE mchResult AS CHARACTER   NO-UNDO.

    IF ipinIndex > 0 AND NUM-ENTRIES(cchAttributes) >= ipinIndex THEN
      mchResult = ENTRY(ipinIndex,cchAttributes,'<') + '="' + ENTRY(ipinIndex,cchAttributesValues,'<') + '"'.
  END METHOD.

  METHOD PUBLIC CHARACTER Desescapar(INPUT ipchData AS CHARACTER):
    /* las entidades ya las escap SAX, excepto &lt; */
    ASSIGN
      ipchData = REPLACE(ipchData,'&lt;','<')
      .
    RETURN ipchData.
  END METHOD.

  /* invocados por SAX (IXMLParser) */
  METHOD PUBLIC LOGICAL DocumentStart():
    IF NOT clgInSAX THEN DO:
      DoLog('XMLParser','Slo SAX puede invocar el mtodo DocumentStart: Se ha invocado un mtodo de interpretacin XML por fuera de la ejecucin de SAX','ERROR').
      RETURN FALSE.
    END.
    ASSIGN 
      cchCurrentTag = ?
      cchCurrentContent = ?
      cchAttributes =  ?
      cchAttributesValues = ?
      cchTagHistory = ''.
    IF VALID-OBJECT(cobWorker) THEN
      cobWorker:WorkStart(THIS-OBJECT).
    RETURN TRUE.
  END METHOD.
  METHOD PUBLIC LOGICAL ElementStart(INPUT ipchNameSpaceURI AS CHARACTER,INPUT ipchLocalName AS CHARACTER,INPUT ipchQName AS CHARACTER,INPUT iphnSaxAttObj AS HANDLE):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.

    IF NOT clgInSAX THEN DO:
      DoLog('XMLParser','Slo SAX puede invocar el mtodo DocumentStart: Se ha invocado un mtodo de interpretacin XML por fuera de la ejecucin de SAX','ERROR').
      RETURN FALSE.
    END.
    ASSIGN 
      cchCurrentTag = ipchLocalName
      cchTagHistory = cchTagHistory + ',' + cchCurrentTag
      cchCurrentContent = ''
      cchAttributes =  ''
      cchAttributesValues = ''.
    IF iphnSaxAttObj:NUM-ITEMS > 0 THEN DO:
      cchAttributes = iphnSaxAttObj:GET-LOCALNAME-BY-INDEX(1).
      cchAttributesValues =  iphnSaxAttObj:GET-VALUE-BY-INDEX(1).
      DO minIndex = 2 TO iphnSaxAttObj:NUM-ITEMS:
        cchAttributes = cchAttributes + '<' + iphnSaxAttObj:GET-LOCALNAME-BY-INDEX(minIndex).
        cchAttributesValues =  cchAttributesValues + '<' + REPLACE(iphnSaxAttObj:GET-VALUE-BY-INDEX(minIndex),'<','&lt;').
      END.
    END.
    IF VALID-OBJECT(cobWorker) THEN
      cobWorker:ElementStart(THIS-OBJECT).
    RETURN TRUE.
  END METHOD.
  METHOD PUBLIC LOGICAL ElementEnd(INPUT ipchNameSpaceURI AS CHARACTER,INPUT ipchLocalName AS CHARACTER,INPUT ipchQName AS CHARACTER):
    DEFINE VARIABLE minEntries AS INTEGER     NO-UNDO.
    IF NOT clgInSAX THEN DO:
      DoLog('XMLParser','Slo SAX puede invocar el mtodo DocumentStart: Se ha invocado un mtodo de interpretacin XML por fuera de la ejecucin de SAX','ERROR').
      RETURN FALSE.
    END.
    /* orden inverso a ElementStart!*/
    IF VALID-OBJECT(cobWorker) THEN
      cobWorker:ElementEnd(THIS-OBJECT).
    minEntries = NUM-ENTRIES(cchTagHistory).
    ASSIGN
      cchCurrentTag = ENTRY(minEntries,cchTagHistory)
      cchTagHistory = SUBSTRING(cchTagHistory,1,LENGTH(cchTagHistory) - LENGTH(cchCurrentTag) - 1)
      minEntries = minEntries - 1
      cchCurrentContent = ''.
    cchCurrentTag = ENTRY(minEntries,cchTagHistory).
    RETURN TRUE.
  END METHOD.
  METHOD PUBLIC LOGICAL Content(INPUT ipchData AS CHARACTER):
    IF NOT clgInSAX THEN DO:
      DoLog('XMLParser','Slo SAX puede invocar el mtodo DocumentStart: Se ha invocado un mtodo de interpretacin XML por fuera de la ejecucin de SAX','ERROR').
      RETURN FALSE.
    END.
    cchCurrentContent = cchCurrentContent + ipchData.
    RETURN TRUE.
  END METHOD.
  METHOD PUBLIC VOID XMLError(INPUT ipchMessage AS CHARACTER):
    IF NOT clgInSAX THEN DO:
      DoLog('XMLParser','Slo SAX puede invocar el mtodo DocumentStart: Se ha invocado un mtodo de interpretacin XML por fuera de la ejecucin de SAX','ERROR').
      RETURN.
    END.
    DoLog('SAX',SUBSTITUTE('Error de interpretacin XML:&1',ipchMessage),'ERROR').
  END METHOD.
  METHOD PUBLIC LOGICAL DocumentEnd():
    IF NOT clgInSAX THEN DO:
      DoLog('XMLParser','Slo SAX puede invocar el mtodo DocumentStart: Se ha invocado un mtodo de interpretacin XML por fuera de la ejecucin de SAX','ERROR').
      RETURN FALSE.
    END.
    IF VALID-OBJECT(cobWorker) THEN
      cobWorker:WorkEnd(THIS-OBJECT).
    RETURN TRUE.
  END METHOD.

  /* invocados por los workers */
  METHOD PUBLIC VOID DoLog(INPUT ipchSource AS CHARACTER, INPUT ipchData AS CHARACTER, INPUT ipchLevel AS CHARACTER):
    IF VALID-OBJECT(cobLogger) THEN
      cobLogger:OutLog(SUBSTITUTE("(&1):&2",ipchSource,ipchData),cobLogger:getLevel(ipchLevel)).
  END METHOD.
  METHOD PUBLIC CHARACTER AttributeValue(INPUT ipchAttribute AS CHARACTER):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    minIndex = LOOKUP(ipchAttribute,cchAttributes,'<').
    IF minIndex EQ 0 THEN
      RETURN ?.
    RETURN Desescapar(ENTRY(minIndex,cchAttributesValues,'<')).
  END METHOD.
  METHOD PUBLIC CHARACTER CurrentElement():
    RETURN cchCurrentTag.
  END METHOD.
  METHOD PUBLIC CHARACTER ParentElement(INPUT ipinLevel AS INTEGER):
    DEFINE VARIABLE minNETH AS INTEGER     NO-UNDO.
    minNETH = NUM-ENTRIES(cchTagHistory).
    IF ipinLevel > minNETH THEN
      RETURN ?.
    ELSE
      RETURN ENTRY(minNETH - ipinLevel,cchTagHistory).
  END METHOD.
  METHOD PUBLIC LOGICAL AttributeExists(INPUT ipchAttName AS CHARACTER):
    RETURN LOOKUP(ipchAttName,cchAttributes,'<') > 0.
  END METHOD.
  METHOD PUBLIC CHARACTER ElementContent():
    RETURN cchCurrentContent.
  END METHOD.

  METHOD PUBLIC IXMLWorker setWorker(INPUT ipobNewWorker AS IXMLWorker):
    cobWorker = ipobNewWorker.
    RETURN ipobNewWorker.
  END METHOD.

  /* ILoggeable */
  METHOD PUBLIC ILogger setLogger(INPUT ipobLogger AS ILogger):
    cobLogger = ipobLogger.
    RETURN ipobLogger.
  END METHOD.
END CLASS.
