/*****************
Propiedad intelectual de Computer Integrity (info@cintegrity.com)
y NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor original: Thomas Mercer-Hursh (thomas@cintegrity.com)
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase que implementa los mtodos de manipulacin del repositorio UML,
para Enterprise Architect. El grueso de la lgica de generacin
corresponde a esta clase.
- ENGLISH -
Implementation of UML repository manipulation methods, for Enterprise
Architect. Most of the generation logic belongs to this class.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING com.nomadesoft.ABLaUML.repository.*.
USING com.nomadesoft.lib.*.
USING PROGRESS.Lang.AppError.

/** esta clase implementa la interfaz al repositorio, para Enterprise Architect **/
CLASS com.nomadesoft.ABLaUML.repository.EAUtils IMPLEMENTS IUtils,ILoggeable:
  DEFINE PUBLIC PROPERTY cobLogger AS CLASS ILogger NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY clgIncremental AS LOGICAL NO-UNDO
    GET.
    PRIVATE SET.

  define PROTECTED temp-table cttObject
    field chType as character
    field chStereotype as character
    field chName as character
    field inObjectID as integer
    field chEAGUID as character
    FIELD inPackage AS INTEGER
    index ixByName is primary chType chStereotype chName inPackage.

  define PROTECTED temp-table cttStereotype
    field chStereotype as character
    field chAppliesTo as character
    field chEAGUID as character
    index ixByStereotype is primary unique chStereotype chAppliesTo.

  DEFINE PUBLIC PROPERTY cobStereotypeMapper AS IMapper NO-UNDO
    GET.
    SET(INPUT ipobMapper AS IMapper):
      cobStereotypeMapper = ipobMapper.
      EMPTY TEMP-TABLE cttObject.
      EMPTY TEMP-TABLE cttStereotype.
      FillStereotypesTT().
      FillObjectsTT().
    END SET.

  define PRIVATE buffer cbfPackage for Repository.t_Package. /* utilizado por el mtodo recursivo EsPadre */

  DEFINE PROTECTED PROPERTY cinPackageID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(package_id_seq)).
    END GET.
    PRIVATE SET(INPUT ipinPID AS INTEGER):
      current-value(PACKAGE_ID_SEQ) = ipinPID.
    END SET.

  DEFINE PROTECTED PROPERTY cinObjectID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(OBJECT_id_seq)).
    END GET.
    PRIVATE SET(INPUT ipinOID AS INTEGER):
      current-value(OBJECT_ID_SEQ) = ipinOID.
    END SET.

  DEFINE PROTECTED PROPERTY cinOperationID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(operationId_seq)).
    END GET.
    PRIVATE SET(INPUT ipinOID AS INTEGER):
      current-value(OPERATIONID_SEQ) = ipinOId.
    END SET.

  DEFINE PROTECTED PROPERTY cinAttributeID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(current-value(ID_SEQ)).
    END GET.
    PRIVATE SET(INPUT ipinAid AS INTEGER):
      current-value(ID_SEQ) = ipinAID.
    END SET.
                                       
  DEFINE PROTECTED PROPERTY cinObjectPropertyID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(propertyId_seq)).
    END GET.
    PRIVATE SET(INPUT ipinPID AS INTEGER):
      CURRENT-VALUE(propertyid_seq) = ipinPID.
    END SET.

  DEFINE PROTECTED PROPERTY cinAttributeTagID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(propertyId_seq)).
    END GET.
    PRIVATE SET(INPUT ipinTid AS INTEGER):
      CURRENT-VALUE(propertyID_seq) = ipinTid.
    END SET.

  DEFINE PROTECTED PROPERTY cinOperationTagID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(propertyId_seq)).
    END GET.
    PRIVATE SET(INPUT ipinTid AS INTEGER):
      CURRENT-VALUE(propertyId_seq) = ipinTId.
    END SET.
    
  DEFINE PROTECTED PROPERTY cinConnectorID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(connector_id_seq)).
    END GET.
    PRIVATE SET(INPUT ipinCId AS INTEGER):
      current-value(CONNECTOR_ID_SEQ) = ipinCID.
    END SET.

  DEFINE PROTECTED PROPERTY cinConnectorTagID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(current-value(propertyid_seq)).
    END GET.
    PRIVATE SET(INPUT ipinTid AS INTEGER):
      CURRENT-VALUE(propertyId_seq) = ipinTid.
    END SET.

  DEFINE PROTECTED PROPERTY cinDiagramID AS INTEGER NO-UNDO 
    GET():
      RETURN INTEGER(CURRENT-VALUE(diagram_id_seq)).
    END GET.
    PRIVATE SET(INPUT ipinDid AS INTEGER):
      current-value(DIAGRAM_ID_SEQ) = ipinDid.
    END SET.

  DEFINE PROTECTED PROPERTY cinDiagramObjectID AS INTEGER NO-UNDO
    GET():
      RETURN INTEGER(CURRENT-VALUE(instance_id_seq)).
    END GET.
    PRIVATE SET(INPUT ipinOId AS INTEGER):
      current-value(INSTANCE_ID_SEQ) = ipinOid.
    END SET.

  METHOD PROTECTED VOID DoLog(INPUT ipchData AS CHARACTER, INPUT ipchLevel AS CHARACTER):
    IF NOT VALID-OBJECT(cobLogger) THEN
      RETURN.
    cobLogger:OutLog(ipchData,cobLogger:getLevel(ipchLevel)).
  END METHOD.

  /* hay algunos campos que por su nombre no se pueden asignar en forma esttica, para asignarlos dinmicamente se utiliza este mtodo */
  METHOD PROTECTED VOID DynAssign(INPUT iphnBuffer AS HANDLE, INPUT ipchField AS CHARACTER, INPUT ipchValue AS CHARACTER):
    DEFINE VARIABLE mhnField AS HANDLE      NO-UNDO.
    IF NOT VALID-HANDLE(iphnBuffer) THEN
      RETURN.
    mhnField = iphnBuffer:BUFFER-FIELD(ipchField).
    IF NOT VALID-HANDLE(mhnField) THEN
      RETURN.
    mhnField:BUFFER-VALUE = ipchValue.
  END METHOD.
  METHOD PROTECTED CHARACTER DynRetrieve(INPUT iphnBuffer AS HANDLE, INPUT ipchField AS CHARACTER):
    DEFINE VARIABLE mhnField AS HANDLE      NO-UNDO.
    IF NOT VALID-HANDLE(iphnBuffer) THEN
      RETURN ?.
    mhnField = iphnBuffer:BUFFER-FIELD(ipchField).
    IF NOT VALID-HANDLE(mhnField) THEN
      RETURN ?.
    RETURN mhnField:BUFFER-VALUE.
  END METHOD.

  METHOD PROTECTED VOID FillStereotypesTT():
    define buffer mbfStereotype for Repository.t_Stereotypes.
    for each mbfStereotype no-lock:
      create cttStereotype.
      assign 
          cttStereotype.chStereotype = ReverseMap(mbfStereotype.Stereotype)
          cttStereotype.chAppliesTo  = mbfStereotype.AppliesTo
          cttStereotype.chEAGUID = mbfStereotype.EA_GUID
          .
      CATCH e AS Progress.Lang.Error:
        DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
        doLog("Load of Stereotypes failed for " + mbfStereotype.Stereotype + "/" + mbfStereotype.AppliesTo,'WARNING').
        do minIndex = 1 to e:NumMessages:
          DoLog(e:getMessage( minIndex ),'NOTICE').
        end.     
        DELETE OBJECT e.
      END CATCH.
    end.
  END METHOD.

  /* se encarga de realizar el mapeo de estereotipos, de forma de poder utilizar cualquier estereotipo sin tener
  que cambiar el cdigo */
  METHOD PROTECTED CHARACTER DoMap(INPUT ipchStMap AS CHARACTER):
    IF VALID-OBJECT(cobStereotypeMapper) THEN
      ipchStMap = cobStereotypeMapper:MapIt(ipchStMap,FALSE).
    RETURN ipchStMap.
  END METHOD.
  METHOD PROTECTED CHARACTER ReverseMap(INPUT ipchStMap AS CHARACTER):
    IF VALID-OBJECT(cobStereotypeMapper) THEN
      ipchStMap = cobStereotypeMapper:MapIt(ipchStMap,TRUE).
    IF ipchStMap EQ ? THEN
      RETURN ''.
    RETURN ipchStMap.
  END METHOD.

  /* llena la Temp-Table con los datos del repositorio */
  METHOD PROTECTED VOID FillObjectsTT():
    define buffer mbfObject for Repository.t_Object.
    FOR EACH mbfObject:
      CREATE cttObject.
      assign
          cttObject.chType = mbfObject.Object_Type
          cttObject.chStereotype = ReverseMap(mbfObject.Stereotype)
          cttObject.chName = mbfObject.Name
          cttObject.inObjectID = mbfObject.Object_ID
          cttObject.chEAGUID = mbfObject.EA_GUID
          cttObject.inPackage = mbfObject.Package_ID WHEN mbfObject.OBJECT_Type NE 'Package'
          cttObject.inPackage = INTEGER(mbfObject.PData1) WHEN mbfObject.OBJECT_Type EQ 'Package'
          .
    END.
  END METHOD.

  /* verifica los valores mximos en las tablas y establece las secuencias si su valor actual es menor */
  METHOD PROTECTED VOID SetUpSequences():
    define buffer mbfAttribute for Repository.t_Attribute.
    define buffer mbfAttributeTag for Repository.t_AttributeTag.
    define buffer mbfConnector for Repository.t_Connector.
    define buffer mbfConnectorTag for Repository.t_ConnectorTag.
    DEFINE BUFFER mbfDiagram FOR Repository.T_Diagram.
    DEFINE BUFFER mbfDiagramLinks FOR Repository.T_DiagramLinks.
    DEFINE BUFFER mbfDiagramObject FOR Repository.T_DiagramObject.
    define buffer mbfObject for Repository.t_Object.
    define buffer mbfObjectProperty for Repository.t_ObjectProperties.
    define buffer mbfOperation for Repository.t_Operation.
    define buffer mbfOperationTag for Repository.t_OperationTag.
    define buffer mbfPackage for Repository.t_Package.

    FOR EACH mbfAttribute BY ID DESC:
      IF cinAttributeID LT ID THEN
        cinAttributeID = ID.
      LEAVE.
    END.
    FOR EACH mbfAttributeTag BY PropertyID DESC:
      IF cinAttributeTagID LT PropertyID THEN
        cinAttributeTagID = PropertyID.
      LEAVE.
    END.
    FOR EACH mbfConnector BY Connector_ID DESC:
      IF cinConnectorID LT Connector_ID THEN
        cinConnectorID = Connector_ID.
      LEAVE.
    END.
    FOR EACH mbfConnectorTag BY PropertyID DESC:
      IF cinConnectorTagID LT PropertyID THEN
        cinConnectorTagID = PropertyID.
      LEAVE.
    END.
    FOR EACH mbfDiagram BY Diagram_ID DESC:
      IF cinDiagramID LT Diagram_ID THEN
        cinDiagramID = Diagram_ID.
      LEAVE.
    END.
    FOR EACH mbfDiagramLinks BY Instance_ID DESC:
      IF cinDiagramObjectID LT Instance_ID THEN
        cinDiagramObjectID = Instance_ID.
      LEAVE.
    END.
    FOR EACH mbfDiagramObject BY Instance_ID DESC:
      IF cinDiagramObjectID LT Instance_ID THEN
        cinDiagramObjectID = Instance_ID.
      LEAVE.
    END.
    FOR EACH mbfObject BY Object_ID DESC:
      IF cinObjectID LT Object_ID THEN
        cinObjectID = Object_ID.
      LEAVE.
    END.
    FOR EACH mbfObjectProperty BY PropertyID DESC:
      IF cinObjectPropertyID LT PropertyID THEN
        cinObjectPropertyID = PropertyID.
      LEAVE.
    END.
    FOR EACH mbfOperation BY OperationID DESC:
      IF cinOperationID LT OperationID THEN
        cinOperationID = OperationID.
      LEAVE.
    END.
    FOR EACH mbfOperationTag BY PropertyID DESC:
      IF cinOperationTagID LT PropertyID THEN
        cinOperationTagID = PropertyID.
      LEAVE.
    END.
    FOR EACH mbfPackage BY Package_ID DESC:
      IF cinPackageID LT Package_ID THEN
        cinPackageID = Package_ID.
      LEAVE.
    END.
  END METHOD.

  METHOD PROTECTED LOGICAL EsPadre(INPUT ipinParentID AS INTEGER, INPUT ipinPackageID AS INTEGER):
    DEFINE VARIABLE minParent AS INTEGER     NO-UNDO.

    /* dado que es recursiva, utilizo el buffer de la clase para no generar una explosin de buffers */
    FIND cbfPackage NO-LOCK WHERE cbfPackage.Package_ID eq ipinPackageID.
    IF cbfPackage.PARENT_ID EQ ipinParentID THEN
      RETURN TRUE.
    IF cbfPackage.PARENT_ID EQ 0 OR cbfPackage.PARENT_ID EQ ? THEN
      RETURN FALSE.
    RETURN EsPadre(ipinParentID,cbfPackage.PARENT_ID).
  END METHOD.

  METHOD PUBLIC LOGICAL AddPackageNotes(input  ipinPackageID as INTEGER,input  ipchNotes as CHARACTER):
    define buffer mbfPackage for Repository.t_Package.
    if ipinPackageID eq ? then
      undo, THROW NEW AppError("Attempt to add notes to unknown package ID" + (IF ipchNotes NE ? THEN ": " + ipchNotes ELSE ''),1010).

    find mbfPackage exclusive-lock
        where mbfPackage.Package_ID = ipinPackageID.
    mbfPackage.Notes = ipchNotes.
    RETURN TRUE.
    CATCH sError AS Progress.Lang.SysError:
      DELETE OBJECT sError.
      RETURN FALSE.
    END CATCH.
  end METHOD.
  METHOD PUBLIC LOGICAL AddPackageAlias(input  ipinPackageID as INTEGER,input  ipchAlias as CHARACTER):
    define buffer mbfPackage for Repository.t_Package.
    if ipinPackageID eq ? then
      undo, THROW NEW AppError("Attempt to add alias to unknown package ID" + (IF ipchAlias NE ? THEN ": " + ipchAlias ELSE ''),1010).

    find mbfPackage NO-LOCK
        where mbfPackage.Package_ID = ipinPackageID.
    find cttObject
        where cttObject.chType eq 'Package'
        and   cttObject.chName eq mbfPackage.NAME
        AND cttObject.inPackage EQ ipinPackageID
        no-error.
    if not available ( cttObject ) THEN
      RETURN FALSE.
    RETURN AddObjectAlias(cttObject.inObjectID,ipchAlias).
    CATCH sError AS Progress.Lang.SysError:
      DELETE OBJECT sError.
      RETURN FALSE.
    END CATCH.
  end METHOD.
  METHOD PUBLIC LOGICAL AddObjectNotes(input  ipinObjectID as INTEGER,input  ipchNotes as character):
    define buffer mbfObject for Repository.t_Object.
    if ipchNotes eq ? then do:
      DoLog("Attempt to add unknown Notes to object " + string(ipinObjectID) ,'WARNING').
      ipchNotes = "".
    end.

    find mbfObject exclusive-lock
        where mbfObject.Object_ID = ipinObjectID.
    mbfObject.Note = ipchNotes.
    RETURN TRUE.
    CATCH sError AS Progress.Lang.SysError:
      DELETE OBJECT sError.
      RETURN FALSE.
    END CATCH.
  end METHOD.
  METHOD PUBLIC LOGICAL AddObjectAlias(input  ipinObjectID as INTEGER,input  ipchAlias as CHARACTER):
    define buffer mbfObject for Repository.t_Object.
    if ipchAlias eq ? then do:
      DoLog("Attempt to add unknown Alias to object " + string(ipinObjectID) ,'WARNING').
      ipchAlias = "".
    end.

    find mbfObject exclusive-lock
        where mbfObject.Object_ID = ipinObjectID.
    DynAssign(buffer mbfObject:handle,'Alias',ipchAlias).
    RETURN TRUE.
    CATCH sError AS Progress.Lang.SysError:
      DELETE OBJECT sError.
      RETURN FALSE.
    END CATCH.
  end METHOD.

  METHOD PROTECTED CHARACTER FindStereotype(input ipchStereotype as character,input ipchAppliesTo as character):
    define buffer mbfStereotype for Repository.t_Stereotypes.
    find cttStereotype
        where cttStereotype.chStereotype eq ipchStereotype
        and   cttStereotype.chAppliesTo  eq ipchAppliesTo
        no-error.
    /* el mapeo de estereotipos puede hacer que dos estereotipos ctt distintos sean el mismo mbf !! */
    if not available ( cttStereotype )
    then DO:
      DoLog("Creating Stereotype = " + ipchStereotype + " to apply to " + ipchAppliesTo ,'NOTICE').        
      DO ON ERROR UNDO, THROW:
        create mbfStereotype.
        assign
            mbfStereotype.EA_GUID = NewEAGUID()
            mbfStereotype.Stereotype = DoMap(ipchStereotype)
            mbfStereotype.AppliesTo  = ipchAppliesTo
            /* mbfStereotype.Description */
            mbfStereotype.MFEnabled = -1
            /* mbfStereotype.MFPath */
            /* mbfStereotype.MeteFile */
            /* mbfStereotype.Style */
            /* mbfStereotype..VisualType */
            .
        CATCH se AS Progress.Lang.SysError:
          /* supongo que se debe a que ya existe el estereotipo mapeado */
          DELETE OBJECT se.
          FIND mbfStereotype NO-LOCK WHERE mbfStereotype.Stereotype EQ DoMap(ipchStereotype)
              AND mbfStereotype.AppliesTo EQ ipchAppliesTo.
        END CATCH.
      END.
      create cttStereotype.
      assign
          cttStereotype.chStereotype = ipchStereotype
          cttStereotype.chAppliesTo  = ipchAppliesTo
          cttStereotype.chEAGUID = mbfStereotype.EA_GUID
          .
    end.
    RETURN cttStereotype.chEAGUID.
  end METHOD.

  METHOD PROTECTED VOID MakeStereotypeXref(input ipchClientType as character,input ipchStereotype as character,input ipchClientGUID as character):
    define variable mchStereotypeGUID as character no-undo.
    define variable mchType as character no-undo.
    DEFINE VARIABLE mchXrefDesc AS CHARACTER   NO-UNDO.
    define buffer mbfXref for Repository.t_xref.

    mchStereotypeGUID = FindStereotype (
        input ipchStereotype,
        input ipchClientType
        ).
    if mchStereotypeGUID eq ?
    then
      undo, THROW NEW AppError("Stereotype = " + ipchStereotype + " not found in spMakeStereotypeXref for Client Type = " + ipchClientType,1010 ).

    ASSIGN
      mchType = GetXrefType( ipchClientType )
      mchXrefDesc = "@STEREO;Name=" + DoMap(ipchStereotype) + ";GUID=" + mchStereotypeGUID + ";@ENDSTEREO"
      .

    FIND mbfXref WHERE mbfXref.NAME EQ 'Stereotypes' AND mbfXref.TYPE EQ mchType
        AND mbfXref.Client EQ ipchClientGUID AND mbfXref.DESCRIPTION EQ mchXrefDesc NO-ERROR.
    IF AVAILABLE mbfXref THEN
      RETURN.
    create mbfXref.
    assign
        mbfXref.XrefID = NewEAGUID()
        mbfXref.Name = "Stereotypes"
        mbfXref.Type = mchType
        mbfXref.Visibility = "Public"
        /* mbfXref.Namespace */
        /* mbfXref.Requirement */
        /* mbfXref.Constraint */
        /* mbfXref.Behavior */
        /* mbfXref.Partition */
        mbfXref.Description = mchXrefDesc
        mbfXref.Client = ipchClientGUID
        mbfXref.Supplier = "<none>"
        /* mbfXref.Link */
        .

  end METHOD.

  METHOD PROTECTED VOID ChangeStereotypeXref(input ipchClientType as character,input ipchOldStereotype as character,input ipchNewStereotype as character,input ipchClientGUID as CHARACTER):
    define variable mchOldStereotypeGUID as character no-undo.
    define variable mchNewStereotypeGUID as character no-undo.
    define variable mchType as character no-undo.
    define buffer mbfXref for Repository.t_xref.

    mchOldStereotypeGUID = FindStereotype (
        input ipchOldStereotype,
        input ipchClientType
        ).
    if mchOldStereotypeGUID eq ?
    then
      undo, THROW NEW AppError("Stereotype = " + ipchOldStereotype + " not found in spChangeStereotypeXref for Client Type = " + ipchClientType,1010 ).
    mchNewStereotypeGUID = FindStereotype (
        input ipchNewStereotype,
        input ipchClientType
        ).      
    if mchNewStereotypeGUID eq ?
    then
      Undo, THROW NEW AppError("Stereotype = " + ipchNewStereotype + " not found in spChangeStereotypeXref for Client Type = " + ipchClientType,1010 ).

    mchType = GetXrefType( ipchClientType ).

    for each mbfXref exclusive-lock
        where mbfXref.Client eq ipchClientGUID
        and   mbfXref.Type EQ mchType
        :
      if mbfXref.Description matches "*" + DoMap(ipchOldStereotype) + "*"
      then do:
        mbfXref.Description = replace( mbfXref.Description, DoMap(ipchOldStereotype), DoMap(ipchNewStereotype) ).    
        mbfXref.Description = replace( mbfXref.Description, mchOldStereotypeGUID, mchNewStereotypeGUID ).
      end.
    end.

  end METHOD.    

  METHOD PUBLIC INTEGER CreateOperation(input  ipinObjectID as INTEGER,input  ipchOperationName as CHARACTER,input  ipchStereotype as CHARACTER):
    define variable mchOperationType as character no-undo.
    define buffer mbfOperation for Repository.t_Operation.
    DEFINE VARIABLE mchStereotype AS CHARACTER   NO-UNDO.

    mchStereotype = DoMap(ipchStereotype).
    FIND mbfOperation WHERE mbfOperation.OBJECT_ID EQ ipinObjectID
        AND mbfOperation.NAME EQ ipchOperationName
        AND mbfOperation.Stereotype EQ mchStereotype NO-ERROR.
    IF AVAILABLE mbfOperation THEN DO:
      IF NOT clgIncremental THEN
        UNDO, THROW NEW AppError(SUBSTITUTE('Intento de crear operacin duplicada &1 para objeto &2',ipchOperationName,ipinObjectID),2929).
    END.
    ELSE DO:
      create mbfOperation.
      assign
          cinOperationID = cinOperationID + 1
          mbfOperation.OperationID = cinOperationID
          mbfOperation.EA_GUID = NewEAGUID()
          mbfOperation.Object_ID = ipinObjectID
          mbfOperation.Name = ipchOperationName
          mbfOperation.Scope = "Public"
          /* mbfOperation.Type */
          /* mbfOperation.ReturnArray */
          mbfOperation.Stereotype = mchStereotype
          /* mbfOperation.IsStatic */
          /* mbfOperation.Concurrency */
          /* mbfOperation.Notes */
          /* mbfOperation.Behavior */
          /* mbfOperation.Abstract */
          /* mbfOperation.GenOption */
          /* mbfOperation.Synchronized */
          /* mbfOperation.Pos */
          /* mbfOperation.Const */
          /* mbfOperation.Style */
          /* mbfOperation.Pure */
          /* mbfOperation.Throws */
          /* mbfOperation.Classifier */
          /* mbfOperation.Code */
          /* mbfOperation.IsRoot */
          /* mbfOperation.IsLeaf */
          /* mbfOperation.IsQuery */
          /* mbfOperation.StateFlags */
          /* mbfOperation.StyleEx */
          .
    END.
    if ipchStereotype ne ""  THEN DO:
      if ipchStereotype eq "PK"
          or ipchStereotype eq "FK"
          or ipchStereotype eq "index"
      then mchOperationType = "OpTable".
      else mchOperationType = "Operation".

       MakeStereotypeXref (
           input mchOperationType,
           input ipchStereotype,
           input mbfOperation.EA_GUID
           ).
    END.
    RETURN mbfOperation.OperationID.
  END METHOD.

  METHOD PROTECTED CHARACTER NewEAGUID():
    return "~{" + guid(generate-uuid) + "~}".
  end METHOD. 

  METHOD PROTECTED CHARACTER GetXrefType(input ipchClientType as character):
    define variable mchType as character no-undo.

    case ipchClientType:
      when "Component" then mchType = "element".
      when "Package" then mchType = "element".
      when "Class" then mchType = "element".
      when "AttribTable" then mchType = "element".
      when "Attribute" then mchType = "element".
      when "OpTable" then mchType = "element".
      when "Operation" then mchType = "element".
      when "Instantiation" then mchType = "connector".
      when "ControlFlow" then mchType = "connector".
      when "Dependency" then mchType = "connector".
      otherwise 
        do:
          DoLog("GetXrefType called with unknown Client Type = " + ipchClientType,'WARNING').
          return ?.
        end.
    end case.    

    mchType = mchType + " property".
    return mchType.      
  end METHOD.      

  METHOD PUBLIC VOID CreateOperationParam(input  ipinOperationID as integer, input  ipchName as character, input  ipchType as character, input  ipchDefault as character,input  ipinPos as integer, input  ipchKind as character,input  ipchStereotype as CHARACTER):
    define buffer mbfOperationParams for Repository.t_OperationParams.

    FIND mbfOperationParams WHERE mbfOperationParams.OperationID EQ ipinOperationID
        AND mbfOperationParams.NAME EQ ipchName NO-ERROR.
    IF AVAILABLE mbfOperationParams THEN DO:
      IF NOT clgIncremental THEN
        UNDO, THROW NEW AppError(SUBSTITUTE('Intento de crear parmetro duplicado &1 para operacin &2',ipinPos,ipinOperationID),2923).
    END.
    ELSE DO:
      create mbfOperationParams.
      assign
          mbfOperationParams.OperationID = ipinOperationID
          mbfOperationParams.EA_GUID = NewEAGUID()
        .
    END.
    ASSIGN
        mbfOperationParams.Name = ipchName 
        mbfOperationParams.Type = ipchType
        mbfOperationParams.Default = ipchDefault
        mbfOperationParams.Notes = "<<" + DoMap(ipchStereotype) + ">>"  
        mbfOperationParams.Pos = ipinPos
        /* mbfOperationParams.Const */
        /* mbfOperationParams.Style */
        mbfOperationParams.Kind = ipchKind
        /* mbfOperationParams.Classifier */
        /* mbfOperationParams.StyleEx = "" */
        .
  end METHOD.

  METHOD PUBLIC VOID CreateOperationPost(input  ipinOperationID as INTEGER, input  ipchCondition as character, input  ipchType as CHARACTER):
    define buffer mbfOperationPost for Repository.t_OperationPosts.

    FIND mbfOperationPost WHERE mbfOperationPost.OperationID EQ ipinOperationID
        AND mbfOperationPost.PostCondition EQ ipchCondition
        AND mbfOperationPost.TYPE EQ ipchType NO-ERROR.
    IF NOT AVAILABLE mbfOperationPost THEN DO:
      create mbfOperationPost.
      assign
          mbfOperationPost.OperationID = ipinOperationID
          mbfOperationPost.PostCondition = ipchCondition
          mbfOperationPost.Type = ipchType
          /* mbfOperationPost.Notes */
          .
    END.
    ELSE IF NOT clgIncremental THEN
      UNDO, THROW NEW AppError(SUBSTITUTE('Intento de crear PostOperacin duplicada &1, para operacin &2',ipchCondition,ipinOperationID),38383).
  end METHOD.

  METHOD PUBLIC INTEGER SetOperationTag(input  ipinOperationID as integer,input  ipchProperty as character,input  ipchValue as CHARACTER):
    define variable mchNote as character no-undo.
    define buffer mbfOperationTag for Repository.t_OperationTag.

    FIND mbfOperationTag WHERE mbfOperationTag.ElementID EQ ipinOperationID
        AND mbfOperationTag.PROPERTY EQ ipchProperty NO-ERROR.
    IF AVAILABLE mbfOperationTag THEN DO:
      IF NOT clgIncremental THEN
        UNDO, THROW NEW AppError(SUBSTITUTE('Intento de crear Tag duplicado &1 para operacin &2',ipchProperty,ipinOperationID),292).
    END.
    ELSE DO:
      create mbfOperationTag.
      assign
          cinOperationTagID  = cinOperationTagID + 1
          mbfOperationTag.PropertyID = cinOperationTagID
          mbfOperationTag.EA_GUID = NewEAGUID()
          mbfOperationTag.ElementID = ipinOperationID
          mbfOperationTag.Property = ipchProperty
        .
    END.
    if length( ipchValue ) gt 255
    then do:
      mchNote = ipchValue.
      ipchValue = "<memo>".
    end.

    ASSIGN
        mbfOperationTag.Notes = mchNote
        .
    DynAssign(buffer mbfOperationTag:handle,'Value',ipchValue).
    RETURN mbfOperationTag.PropertyID.
  end METHOD.

  METHOD PUBLIC INTEGER SetConnectorTag(input  ipinConnectorID as integer,input  ipchProperty as character,input  ipchValue as character):
    define variable mchNote as character no-undo.
    define buffer mbfConnectorTag for Repository.t_ConnectorTag.

    if length( ipchValue ) gt 255
    then do:
      mchNote = ipchValue.
      ipchValue = "<memo>".
    end.

    FIND mbfConnectorTag EXCLUSIVE-LOCK
      WHERE mbfConnectorTag.ElementID EQ ipinConnectorID
      AND mbfConnectorTag.PROPERTY EQ ipchProperty NO-ERROR.
    IF AVAILABLE mbfConnectorTag THEN DO:
      IF NOT clgIncremental THEN
        UNDO, THROW NEW AppError(SUBSTITUTE('Intento de crear Tag duplicado &1 para conector &2',ipchProperty,ipinConnectorID),292).
    END.
    ELSE DO:
      create mbfConnectorTag.
      assign
          cinConnectorTagID  = cinConnectorTagID + 1
          mbfConnectorTag.EA_GUID = NewEAGUID()
          mbfConnectorTag.PropertyID = cinConnectorTagID
          mbfConnectorTag.ElementID = ipinConnectorID
          mbfConnectorTag.Property = ipchProperty
          .
    END.
    mbfConnectorTag.Notes = mchNote.
    DynAssign(buffer mbfConnectorTag:handle,'Value',ipchValue).
    RETURN mbfConnectorTag.PropertyID.
  end METHOD.

  METHOD PUBLIC INTEGER SetAttributeTag(input  ipinAttributeID as INTEGER,input  ipchProperty as character,input  ipchValue as character):
    define variable mchNote as CHARACTER INITIAL ? no-undo.
    define buffer mbfAttributeTag for Repository.t_AttributeTag.

    if length( ipchValue ) gt 255
    then do:
      mchNote = ipchValue.
      ipchValue = "<memo>".
    end.

    FIND mbfAttributeTag EXCLUSIVE-LOCK
      WHERE mbfAttributeTag.ElementID EQ ipinAttributeID
      AND mbfAttributeTag.PROPERTY EQ ipchProperty NO-ERROR.
    IF AVAILABLE mbfAttributeTag THEN DO:
      IF NOT clgIncremental THEN
        UNDO, THROW NEW AppError(SUBSTITUTE('Intento de crear Tag duplicado &1 para atributo &2',ipchProperty,ipinAttributeID),292).
    END.
    ELSE DO:
      create mbfAttributeTag.
      assign
          cinAttributeTagID  = cinAttributeTagID + 1
          mbfAttributeTag.PropertyID = cinAttributeTagID
          mbfAttributeTag.EA_GUID = NewEAGUID()
          mbfAttributeTag.ElementID = ipinAttributeID
          mbfAttributeTag.Property = ipchProperty
        .
    END.
    mbfAttributeTag.Notes = mchNote.
    DynAssign(buffer mbfAttributeTag:handle,'Value',ipchValue).
    RETURN mbfAttributeTag.PropertyID.
  end METHOD.       

  /* IUtils */
  METHOD PUBLIC INTEGER CreatePackage(input  ipchPackageName as character,input  ipinParentPackageID as integer):
    define buffer mbfPackage for Repository.t_Package.
    for each mbfPackage NO-LOCK
        where mbfPackage.Parent_ID eq ipinParentPackageID
        ON ERROR undo, THROW:
      if mbfPackage.Name eq ipchPackageName THEN DO:
          IF clgIncremental THEN
              RETURN mbfPackage.Package_ID.
          undo, THROW NEW appError( "Attempt to add package " + ipchPackageName + " with parent " + string(ipinParentPackageID) + ", but a package of that name and parent already exists",1010 ).
      END.
    end.
    
    if ipinParentPackageID eq ? then
      undo, THROW NEW AppError("Attempt to create package " + ipchPackageName + " with unknown parent ID",1010 ).
      
    create mbfPackage.
    assign
        cinPackageID = cinPackageID + 1
        mbfPackage.EA_GUID = NewEAGUID()
        mbfPackage.Package_ID = cinPackageID
        mbfPackage.Name = ipchPackageName
        mbfPackage.Parent_ID = ipinParentPackageID
        mbfPackage.CreatedDate = now     
        mbfPackage.ModifiedDate = now   
        /* mbfPackage.Notes = */
        /* XmbfPackage.MLPath = "" */
        mbfPackage.IsControlled = 0
        mbfPackage.LastLoadDate = now   
        mbfPackage.LastSaveDate = now   
        mbfPackage.Version = "1.0"
        /* mbfPackage.Protected = 0 */
        /* mbfPackage.PkgOwner = "" */
        /* mbfPackage.UMLVersion = "2.1" */
        /* mbfPackage.UseDTD = 0 */
        /* mbfPackage.LogXML = 0 */
        /* mbfPackage.CodePath = "" */
        /* mbfPackage.NameSpace = "" */
        /* mbfPackage.Tpos = n */
        /* mbfPackage.PackageFlags = '' */
        /* mbfPackage.BatchSave = 0 */
        /* mbfPackage.BatchLoad = 0 */
        .
    RETURN cinPackageID.
  END METHOD.
  METHOD PUBLIC INTEGER CreateModelPackage(input  ipchPackageName as character, input  ipinParentPackageID as INTEGER, INPUT ipinTipo AS INTEGER):
    DEFINE VARIABLE minPackageID AS INTEGER     NO-UNDO.
    define buffer mbfPackage for Repository.t_Package.
    minPackageID = CreatePackage(ipchPackageName,ipinParentPackageID).
    CreateObject("Package",ipchPackageName,minPackageID,ipinParentPackageID,'','','').
    
    FIND mbfPackage EXCLUSIVE-LOCK where mbfPackage.Package_ID eq minPackageID.
    IF ipinTipo GT 0 THEN
      mbfPackage.PackageFlags = 'isModel=1;VICON=' + STRING(ipinTipo).
    ELSE
      mbfPackage.PackageFlags = 'isModel=1'.
    RETURN minPackageID.
  END METHOD.
  METHOD PUBLIC LOGICAL DeletePackage(INPUT ipinPackageId AS INTEGER,INPUT iplgObjectToo AS LOGICAL):
    define buffer mbfPackage for Repository.t_Package.
    define buffer mbfObject for Repository.t_Object.
    FIND mbfPackage EXCLUSIVE-LOCK WHERE mbfPackage.Package_ID EQ ipinPackageId NO-ERROR.
    IF NOT AVAILABLE mbfPackage THEN
      RETURN FALSE.
    IF iplgObjectToo THEN DO:
      FIND mbfObject EXCLUSIVE-LOCK WHERE mbfObject.Package_ID EQ ipinPackageId AND mbfObject.NAME EQ mbfPackage.NAME NO-ERROR.
      IF NOT AVAILABLE mbfObject THEN
        RETURN FALSE.
      DELETE mbfObject.
    END.
    DELETE mbfPackage.
    RETURN TRUE.
  END METHOD.
  METHOD PUBLIC VOID MovePackage(INPUT ipchPackageName AS CHARACTER, INPUT ipinParentPackageID as integer, input  ipinNewParentPackageID as INTEGER):
    define buffer mbfPackage for Repository.t_Package.
    DEFINE BUFFER mbfObject FOR Repository.T_Object.
    FIND mbfPackage WHERE (mbfPackage.Parent_ID eq ipinParentPackageID OR ipinParentPackageID EQ ?)
        AND mbfPackage.Name eq ipchPackageName EXCLUSIVE-LOCK NO-ERROR.
    IF AMBIGUOUS mbfPackage THEN
      UNDO, THROW NEW AppError("Paquete ambiguo",2929).
    ELSE IF NOT AVAILABLE mbfPackage THEN
      UNDO, THROW NEW AppError("El paquete no existe",29292).
    mbfPackage.Parent_ID = ipinNewParentPackageID.
    FIND mbfObject EXCLUSIVE-LOCK WHERE mbfObject.PData1 EQ STRING(mbfPackage.Package_ID) AND mbfObject.NAME EQ ipchPackageName AND mbfObject.OBJECT_Type EQ 'Package' NO-ERROR.
    IF AVAILABLE mbfObject THEN
      mbfObject.Package_ID = ipinNewParentPackageID.
  END METHOD.
  METHOD PUBLIC VOID MoveObject(INPUT ipinObjectID AS INTEGER, input  ipinNewParentPackageID as INTEGER):
    DEFINE BUFFER mbfObject FOR Repository.T_Object.

    FIND mbfObject WHERE mbfObject.OBJECT_ID EQ ipinObjectID EXCLUSIVE-LOCK NO-ERROR.
    IF NOT AVAILABLE mbfObject THEN
      RETURN.
    IF mbfObject.OBJECT_Type EQ 'Package' THEN DO: /* tengo que enonctrar el padre del nuevo paquete */
      DEFINE BUFFER mbfPackage FOR Repository.T_Package.
      FIND mbfPackage NO-LOCK WHERE mbfPackage.PARENT_ID EQ ipinNewParentPackageID AND mbfPackage.NAME EQ mbfObject.NAME.
      mbfObject.Package_ID = mbfPackage.Parent_ID.
      mbfObject.PData1 = STRING(ipinNewParentPackageID).
    END.
    ELSE
      mbfObject.Package_ID = ipinNewParentPackageID.
    FIND cttObject WHERE cttObject.inObjectID EQ ipinObjectID.
    cttObject.inPackage = ipinNewParentPackageID.
  END METHOD.
  METHOD PUBLIC INTEGER PackageParent(INPUT ipchPackageName AS CHARACTER):
    define buffer mbfPackage for Repository.t_Package.
    FOR EACH mbfPackage no-lock
      WHERE mbfPackage.Name eq ipchPackageName:
      RETURN mbfPackage.Parent_ID.
    END.
    RETURN ?.
  END METHOD.
  METHOD PUBLIC INTEGER FindPackage(input  ipchPackageName as character, input  ipinParentPackageID as INTEGER, INPUT iplgRealChild AS LOGICAL):
    /* EsPadre utiliza el buffer de la clase, por lo cual necesito definir mi propio buffer */
    define buffer mbfPackage for Repository.t_Package. 
  
    FOR EACH mbfPackage no-lock
        where mbfPackage.Name EQ ipchPackageName:
      IF ipinParentPackageID EQ ? OR mbfPackage.Parent_ID EQ ipinParentPackageID THEN
        RETURN mbfPackage.Package_ID.
    END.
    /* si no se encontr, entonces ver si hay alguno que est en niveles inferiores al nivel de hijo */
    IF ipinParentPackageID NE ? AND NOT iplgRealChild THEN
      FOR EACH mbfPackage no-lock
          where mbfPackage.Name EQ ipchPackageName:
        IF EsPadre(ipinParentPackageID,mbfPackage.Package_ID) THEN
          RETURN mbfPackage.Package_ID.
      END.
    RETURN 0.
  END METHOD.
  METHOD PUBLIC INTEGER FindPackage(input  ipchPackageName as character, input  ipinParentPackageID as INTEGER):
    RETURN FindPackage(ipchPackageName,ipinParentPackageID,FALSE).
  END METHOD.

  /* manipulacin de objetos */
  METHOD PUBLIC INTEGER CreateObject(input  ipchObjectType as character,input  ipchObjectName as character,input  ipinPackageID as INTEGER, input  ipinParentID as INTEGER,input  ipchStereotype as character,input  ipchNotes as CHARACTER,input ipchAlias as CHARACTER):
    define variable mchEAGUID as character no-undo.
    define variable mhnBuffer as handle no-undo.
    define variable mhnField as handle no-undo.
    define buffer mbfObject for Repository.t_Object.

    if ipchAlias eq ? then do:
      DoLog("Attempt to add unknown Alias to object " + ipchObjectName + " in spCreateObject",'WARNING').
      ipchAlias = "".
    end.
    if ipchNotes eq ? then do:
      DoLog("Attempt to add unknown Notes to object " + ipchObjectName + " in spCreateObject" ,'WARNING' ).
      ipchNotes = "".
    end.
  
    DoLog(SUBSTITUTE("Find Object &1, type &2, with package &3",ipchObjectName,ipchObjectType,ipinPackageID),'DEBUG').
    find cttObject
      where cttObject.chType eq ipchObjectType
      and   cttObject.chStereotype eq ipchStereotype
      and   cttObject.chName eq ipchObjectName
      AND cttObject.inPackage EQ ipinPackageID
      no-error.
    IF AMBIGUOUS(cttObject) THEN
      DoLog(SUBSTITUTE("Abmiguous: &1, &2",ipchObjectName,ipinPackageID),'DEBUG').
    ELSE IF AVAILABLE(cttObject) THEN
      DoLog(SUBSTITUTE("Found object: &1, &2",ipchObjectName,ipinPackageID),'DEBUG').
    if not available( cttObject ) then do:
      DoLog(SUBSTITUTE("Object creation: &1, &2, &3",cinObjectID,ipchObjectName,ipinPackageID),'DEBUG').
      create mbfObject.
      assign
          cinObjectID = cinObjectID + 1
          mbfObject.EA_GUID = NewEAGUID()
          mbfObject.Object_ID = cinObjectID
          mbfObject.Name = ipchObjectName
          mbfObject.Author = "com.nomadesoft.ABLtoUML.repository.EAUtils"
          mbfObject.Version = "1.0"
          mbfObject.Note = ""
          mbfObject.Package_ID = ipinPackageID
          mbfObject.Package_ID = ipinParentID WHEN (ipchObjectType = "Package" AND ipinParentID NE ?)
          mbfObject.Package_ID = 0 WHEN (ipchObjectType = "Package" AND ipinParentID EQ ?)
          mbfObject.Stereotype = DoMap(ipchStereotype)
          mbfObject.Ntype = 19 WHEN (ipchObjectType EQ 'Text' AND ipchStereotype EQ '')
          /* mbfObject.Effort */
          /* mbfObject.Style */
          mbfObject.Backcolor = -1
          mbfObject.BorderStyle = 0
          mbfObject.Borderwidth = -1
          mbfObject.FontColor = -1
          mbfObject.Bordercolor = -1         
          mbfObject.CreatedDate = now         
          mbfObject.Status = "Implemented"          
          /* mbfObject.Abstract = 0 */          
          /* mbfObject.Tagged */
          /* mbfObject.Pdata5 */
          /* mbfObject.Concurrency */
          /* mbfObject.Visibility */
          /* mbfObject.Persistence */
          /* mbfObject.Cardinality */
          mbfObject.Gentype = "OpenEdge ABL"
          /* mbfObject.Genfile */
          /* mbfObject.Header1 */
          /* mbfObject.Header2 */
          mbfObject.Phase = "1.0"
          mbfObject.Scope = "Public"
          /* mbfObject.Genoption */
          /* mbfObject.Genlinks */
          mbfObject.Classifier = 0 WHEN (ipchObjectType = "Text" AND ipchStereotype EQ '')
          /* mbfObject.Runstate */
          /* mbfObject.Classifier_GUID */
          /* mbfObject.Tpos */
          /* mbfObject.IsRoot */
          /* mbfObject.IsLeaf */
          /* mbfObject.IsActive */
          /* mbfObject.StateFlags */
          /* mbfObject.Multiplicity */
          /* mbfObject.StyleEx */
          /* mbfObject.ActionFlags */
          /* mbfObject.EventFlags */
          .
      create cttObject.
      assign
          cttObject.chName = ipchObjectName
          cttObject.inObjectID = cinObjectID
          cttObject.chEAGUID = mbfObject.EA_GUID
          .
    end.
    ELSE
      FIND mbfObject EXCLUSIVE-LOCK WHERE mbfObject.OBJECT_ID EQ cttObject.inObjectID.
    assign
        mbfObject.Object_Type = ipchObjectType
        /* mbfObject.Diagram_ID */
        /* mbfObject.Alias = ipchAlias  no se puede agregar a kwf.lst porque explota todo !! */
        mbfObject.ModifiedDate = now
        /* mbfObject.Abstract = 0 */          
        /* mbfObject.Tagged */
        mbfObject.Gentype = "OpenEdge" WHEN ipchStereotype EQ 'nsTable'
        mbfObject.Pdata1 = string(ipinPackageID) when (ipchObjectType = "Package" or (ipchObjectType = "Text" AND ipchStereotype NE ''))
        mbfObject.Pdata1 = "0" when (ipchObjectType = "Text" AND ipchStereotype EQ '')
        mbfObject.Pdata2 = "OpenEdge" when ipchStereoType = "nsTable"
        mbfObject.Pdata3 = "0" when ipchObjectType = "Text"
        mbfObject.Pdata4 = "0" when ipchStereoType = "nsTable"
        mbfObject.ParentID = ipinParentID WHEN (ipchObjectType NE "Package")
        .
    DynAssign(BUFFER mbfObject:HANDLE,'Alias',ipchAlias).
    assign
        cttObject.chType = ipchObjectType
        cttObject.chStereotype = ipchStereotype
        cttObject.inPackage = ipinPackageID /* mbfObject.Package_ID not allways!! */
        .
    if ipchStereotype ne "" THEN
      MakeStereotypeXref (
          input ipchObjectType,
          input ipchStereotype,
          input cttObject.chEAGUID
          ).
    RETURN mbfObject.OBJECT_ID.
  END METHOD.
  METHOD PUBLIC INTEGER FindObject(input  ipchObjectType as character,input ipchStereotype as character,input  ipchObjectName as character,INPUT ipinPackage AS INTEGER):
    IF ipinPackage EQ ? THEN
      find cttObject
          where cttObject.chType eq ipchObjectType
          and   cttObject.chStereotype eq ipchStereotype
          and   cttObject.chName eq ipchObjectName
          no-error.
    ELSE
      find cttObject
          where cttObject.chType eq ipchObjectType
          and   cttObject.chStereotype eq ipchStereotype
          and   cttObject.chName eq ipchObjectName
          AND cttObject.inPackage EQ ipinPackage
          no-error.
    if ambiguous ( cttObject )
    then
      UNDO, THROW NEW AppError( "More than one object with type = " + ipchObjectType + ", stereotype = " + ipchStereotype + " and name = " + ipchObjectName,1010 ).
    if not available ( cttObject ) THEN
      /* Handle in caller
      UNDO, THROW NEW AppError("No object with type = " + ipchObjectType + ", stereotype = " + ipchStereotype + " and name = " + ipchObjectName + " found" ).
      */
      RETURN ?.
    RETURN cttObject.inObjectID.
  END METHOD.
  METHOD PUBLIC INTEGER CreateAttribute(input ipinObjectID as INTEGER,input ipchObjectName as character,input ipchStereotype as character,input ipchNotes as character ,input ipinPos as INTEGER,input ipinNotNull as integer,input ipchDefault as character,input ipchDataType as CHARACTER,input ipchAttributeType as CHARACTER):
    define variable ichEAGUID as character no-undo.
    define buffer mbfAttribute for Repository.t_Attribute.
    DEFINE VARIABLE mchStereotype AS CHARACTER   NO-UNDO.

    mchStereotype = DoMap(ipchStereotype).
  
    FIND mbfAttribute WHERE mbfAttribute.NAME EQ ipchObjectName
        AND mbfAttribute.OBJECT_ID EQ ipinObjectID NO-ERROR.
    IF AVAILABLE mbfAttribute THEN DO:
      IF NOT clgIncremental THEN
        UNDO, THROW NEW AppError(SUBSTITUTE("Ya existe un atributo &1 para el objeto &2",ipchObjectName,ipinObjectID),292).
    END.
    ELSE DO:
      create mbfAttribute.
      assign
          cinAttributeID = cinAttributeID + 1
          mbfAttribute.EA_GUID = NewEAGUID()
          mbfAttribute.Object_ID = ipinObjectID
          mbfAttribute.Name = ipchObjectName
          mbfAttribute.Scope = "Public"
          mbfAttribute.Stereotype = mchStereotype
          /* mbfAttribute.Containment */
          /* mbfAttribute.IsStatic =                  IsUnique    How to set */
          /* mbfAttribute.IsCollection            Source of FK    How to set */
          /* mbfAttribute.IsOrdered               Is PK           How to set */ 
          mbfAttribute.AllowDuplicates = ipinNotNull
          /* mbfAttribute.LowerBound */
          /* mbfAttribute.UpperBound */
          /* mbfAttribute.Container */
          mbfAttribute.Notes = ipchNotes
          /* mbfAttribute.Derived */
          mbfAttribute.ID = cinAttributeID
          mbfAttribute.Pos = ipinPos
          /* mbfAttribute.GenOption */
          /* mbfAttribute.Length */
          /* mbfAttribute.Precision */
          /* mbfAttribute.Scale */
          /* mbfAttribute.Const */
          /* mbfAttribute.Classifier */
          mbfAttribute.Default = ipchDefault
          mbfAttribute.Type = ipchDataType
          /* mbfAttribute.StyleEx */
          .
    END.
    if ipchStereotype ne "" THEN
      MakeStereotypeXref (
          input ipchAttributeType,
          input ipchStereotype,
          input mbfAttribute.EA_GUID
          ).
    RETURN mbfAttribute.ID.
  END METHOD.
  METHOD PUBLIC LOGICAL SetAttributeAlias(input  ipinAttributeID as INTEGER,input  ipchAlias AS CHARACTER):
    define buffer mbfAttribute for Repository.t_Attribute.

    FIND mbfAttribute WHERE mbfAttribute.ID EQ ipinAttributeID NO-ERROR.
    IF AVAILABLE mbfAttribute THEN DO:
      assign
          mbfAttribute.Style = ipchAlias
          mbfAttribute.StyleEx = 'IsLiteral=0;volatile=0;'
          .
    END.
    RETURN FALSE.
  END METHOD.

  METHOD PUBLIC INTEGER SetObjectTag(input  ipinObjectID as integer,input  ipchProperty as CHARACTER,input  ipchValue as CHARACTER):
    define variable mchNote as character no-undo.
    define buffer mbfObjectProperty for Repository.t_ObjectProperties.

    if length( ipchValue ) gt 255
    then do:
      mchNote = ipchValue.
      ipchValue = "<memo>".
    end.

    find mbfObjectProperty exclusive-lock
        where mbfObjectProperty.Object_ID = ipinObjectID
        and   mbfObjectProperty.Property = ipchProperty
        no-error.
    if not available ( mbfObjectProperty )
    then do:
      create mbfObjectProperty.  
      assign
          cinObjectPropertyID  = cinObjectPropertyID + 1
          mbfObjectProperty.PropertyID = cinObjectPropertyID
          mbfObjectProperty.EA_GUID = NewEAGUID()
          mbfObjectProperty.Object_ID = ipinObjectID
          mbfObjectProperty.Property = ipchProperty
          mbfObjectProperty.Notes = mchNote
          .  
    end.
    /* ibfObjectProperty.Value */
    DynAssign(buffer mbfObjectProperty:handle,'Value',ipchValue).
    RETURN mbfObjectProperty.PropertyID.
  END METHOD.
  METHOD PUBLIC CHARACTER SetConnectorAlias(INPUT ipinConnectorID AS INTEGER, INPUT ipchAlias AS CHARACTER):
    DEFINE VARIABLE mchPrevStyle AS CHARACTER   NO-UNDO.
    define buffer mbfConnector for Repository.t_Connector.
    FIND mbfConnector WHERE mbfConnector.Connector_ID EQ ipinConnectorID EXCLUSIVE-LOCK.
    ASSIGN
      mchPrevStyle = mbfConnector.StyleEx
      mbfConnector.StyleEx = 'alias=' + ipchAlias + ';'
      .
    IF ipchAlias EQ '' OR ipchAlias EQ ? OR mchPrevStyle NE ? THEN
      DoLog(SUBSTITUTE('Connector &1 alias: &2 (&3)',ipinConnectorID,ipchAlias,mchPrevStyle),'DEBUG').
    RETURN mchPrevStyle.
  END METHOD.
  METHOD PUBLIC INTEGER CreateConnector(input  ipchConnectorType as character,input  ipchSourceCardinality as character,input  ipchDestinationCardinality as CHARACTER,input  ipchSourceRole as character,input  ipchDestinationRole as CHARACTER,input  ipinStartObjectID as INTEGER,input  ipinEndObjectID as integer,input  ipchStereotype as CHARACTER,INPUT iplgForce AS LOGICAL):
    DEFINE VARIABLE mchDBstereotype AS CHARACTER   NO-UNDO.
    define buffer mbfConnector for Repository.t_Connector.
    if     ipinStartObjectID eq ?
        or ipinStartObjectID eq 0
        or ipinEndObjectID eq ?
        or ipinEndObjectID eq 0
    then
      undo, THROW NEW AppError(substitute("Cannot make &1 connection from object &2 to object &3 (unknown endpoint)", ipchConnectorType, ipinStartObjectID, ipinEndObjectID),1010 ).

    mchDBstereotype = DoMap(ipchStereotype).
    for each mbfConnector exclusive-lock
        where mbfConnector.Start_Object_ID eq ipinStartObjectID
        and   mbfConnector.End_Object_ID eq ipinEndObjectID
        and   mbfConnector.Connector_Type eq ipchConnectorType
        :
      if mbfConnector.Stereotype eq mchDBStereotype
      then do:
        DoLog(substitute("Connector already exists for type &1 connection from object &2 to object &3", ipchConnectorType, ipinStartObjectID, ipinEndObjectID),'DEBUG' ).
        IF iplgForce THEN
          LEAVE.
        assign
            mbfConnector.SourceCard = ipchSourceCardinality
            mbfConnector.DestCard = ipchDestinationCardinality
            mbfConnector.SourceRole = ipchSourceRole            /* For FK Links */
            mbfConnector.DestRole = ipchDestinationRole               /* For PK */
            .
        RETURN mbfConnector.Connector_ID.
      end.
    end.    

    create mbfConnector.
    assign
        cinConnectorID = cinConnectorID + 1
        mbfConnector.Connector_ID = cinConnectorID
        mbfConnector.EA_GUID = NewEAGUID()
        /* mbfConnector.Name */
        mbfConnector.Direction = "Source -> Destination"
        /* mbfConnector.Notes */
        mbfConnector.Connector_Type = ipchConnectorType
        /* mbfConnector.SubType */
        mbfConnector.SourceCard = ipchSourceCardinality
        mbfConnector.SourceAccess = "Public"
        /* mbfConnector.SourceElement */
        mbfConnector.DestCard = ipchDestinationCardinality
        mbfConnector.DestAccess = "Public"
        /* mbfConnector.DestElement */
        mbfConnector.SourceRole = ipchSourceRole                /* For FK Links */
        /* mbfConnector.SourceRoleType */
        /* mbfConnector.SourceRoleNote */
        mbfConnector.SourceContainment = "Unspecified"
        mbfConnector.SourceIsAggregate = 0
        mbfConnector.SourceIsOrdered = 0
        /* mbfConnector.SourceQualifier */
        mbfConnector.DestRole = ipchDestinationRole                   /* For PK */
        /* mbfConnector.DestRoleType */
        /* mbfConnector.DestRoleNote */
        mbfConnector.DestContainment = "Unspecified"
        mbfConnector.DestIsAggregate = 0
        mbfConnector.DestIsOrdered = 0
        /* mbfConnector.DestQualifier */
        mbfConnector.Start_Object_ID = ipinStartObjectID
        mbfConnector.End_Object_ID = ipinEndObjectID
        /* mbfConnector.Top_Start_Label */
        /* mbfConnector.Top_Mid_Label */
        /* mbfConnector.Top_End_Label */
        /* mbfConnector.Btm_Start_Label */
        /* mbfConnector.Btm_Mid_Label */
        /* mbfConnector.Btm_End_Label */
        /* mbfConnector.Start_Edge */
        /* mbfConnector.End_Edge */
        /* mbfConnector.PTStartX */
        /* mbfConnector.PTStartY */
        /* mbfConnector.PTEndX */
        /* mbfConnector.PEEdnY */
        /* mbfConnector.SeqNo */
        /* mbfConnector.HeadStyle */
        /* mbfConnector.LineStyle */
        /* mbfConnector.RouteStyle */
        /* mbfConnector.IsBold */
        /* mbfConnector.LineColor */
        mbfConnector.StereoType = mchDBStereotype
        /* mbfConnector.VirtualInheritance */
        /* mbfConnector.LinkAccess */
        /* mbfConnector.Pdata1 */
        /* mbfConnector.Pdata2 */
        /* mbfConnector.Pdata3 */
        /* mbfConnector.Pdata4 */
        /* mbfConnector.Pdata5 */    /* Applies to Data Table Links */
        /* mbfConnector.DiagramID */
        /* mbfConnector.SourceConstraint */
        /* mbfConnector.DestConstraint */
        /* mbfConnector.SourceIsNavigable */
        /* mbfConnector.DestIsNavigable */
        /* mbfConnector.IsRoot */
        /* mbfConnector.IsLeaf */
        /* mbfConnector.IsSpec */
        mbfConnector.SourceChangeable = "none"
        mbfConnector.DestChangeable = "none"
        mbfConnector.SourceTS = "instance"
        mbfConnector.DestTS = "instance"
        /* mbfConnector.StateFlags */
        /* mbfConnector.ActionFlags */
        /* mbfConnector.IsSignal */
        /* mbfConnector.IsStimulus */
        /* mbfConnector.DispatchAction */
        /* mbfConnector.Target2 */
        /* mbfConnector.StyleEx */
        /* mbfConnector.SourceStereotype */
        /* mbfConnector.DestStereotype */
        mbfConnector.SourceStyle = "Union=0;Derived=0;AllowDuplicates=0;Owned=0;Navigable=Non-Navigable;" 
        mbfConnector.DestStyle = "Union=0;Derived=0;AllowDuplicates=0;Owned=0;Navigable=Navigable;"
        /* mbfConnector.EventFlags */
        .
    if ipchStereotype ne "" THEN
      MakeStereotypeXref (
          input ipchConnectorType,
          input ipchStereotype,
          input mbfConnector.EA_GUID
          ).

    RETURN mbfConnector.Connector_ID.
    CATCH e AS Progress.Lang.Error:
      DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
      DoLog(substitute("Creation of Connector failed for &1 connection from object &2 to object &3", ipchConnectorType, ipinStartObjectID, ipinEndObjectID),'ERROR' ).
      do minIndex = 1 to e:numMessages:
        DoLog(e:getMessage( minIndex ),'NOTICE' ).
      end.
      UNDO, THROW NEW AppError("Creation of Connector failed",1010).     
    end.
  END METHOD.
  METHOD PUBLIC INTEGER SetDataLink(input ipinStartObjectID as INTEGER,input ipinEndObjectID as INTEGER,input iplgHasCreate as LOGICAL,input iplgHasRead as logical,input iplgHasUpdate as LOGICAL,input iplgHasDelete as logical,input iplgIsSummary as LOGICAL,INPUT ipchConnAlias AS CHARACTER):
    define variable mchStereotype as character no-undo.
    define variable minConnectorID as integer    no-undo.
    define variable mchNewStereotype as character no-undo.
    define variable minConnectorTagID as integer    no-undo.
    define variable mchFileMode as character  no-undo.
    define buffer mbfConnector for Repository.t_Connector.
    define buffer mbfConnectorTag for Repository.t_ConnectorTag.

    for each mbfConnector exclusive-lock
        where mbfConnector.Start_Object_ID eq ipinStartObjectID
        and   mbfConnector.End_Object_ID eq ipinEndObjectID
        and   mbfConnector.Connector_Type eq "Dependency"
        :
      IF INDEX(mbfConnector.StyleEx,'alias=' + ipchConnAlias + ';') EQ 0 THEN
        NEXT.
      mchStereotype = ReverseMap(mbfConnector.Stereotype).
      if mchStereotype begins "nsDataLink"
      then do: 
        if     ( iplgIsSummary and mchStereotype matches "*Summary" )
            or ( not iplgIsSummary and not mchStereotype matches "*Summary" )
        then leave.
      end.
    end.     
    
    if iplgIsSummary 
    then mchStereotype = "nsDataLinkReadOnlySummary".
    else mchStereotype = "nsDataLinkReadOnly". 
  
    if not available ( mbfConnector ) then do:
      minConnectorID = CreateConnector (
          input "Dependency",
          input "",
          input "",
          input "",
          input "",
          input ipinStartObjectID,
          input ipinEndObjectID,
          input mchStereotype,
          ipchConnAlias NE ? AND ipchConnAlias NE ''
          ).
      SetConnectorAlias(minConnectorID,ipchConnAlias).
      find mbfConnector exclusive-lock
          where mbfConnector.Connector_ID = minConnectorID.
    end.
    
    if  ReverseMap(mbfConnector.Stereotype) begins "nsDataLinkReadOnly"
        and ( iplgHasUpdate or iplgHasCreate or iplgHasDelete )
    then do:
      if iplgIsSummary 
      then mchNewStereotype = "nsDataLinkModifySummary".
      else mchNewStereotype = "nsDataLinkModify".     
      ChangeStereotypeXref (
          input "Dependency", 
          input ReverseMap(mbfConnector.Stereotype),
          input mchNewStereotype,
          input mbfConnector.EA_GUID
          ).
     mbfConnector.Stereotype = DoMap(mchNewStereotype).
    end.
    
    minConnectorID = mbfConnector.Connector_ID.
    
    find mbfConnectorTag exclusive-lock
        where mbfConnectorTag.ElementID eq minConnectorID
        and   mbfConnectorTag.Property eq "oeFileMode"
        no-error.
    if not available ( mbfConnectorTag )
    then do:
      minConnectorTagID = SetConnectorTag (
          input minConnectorID,
          input "oeFileMode",
          input "____"
          ).
      find mbfConnectorTag exclusive-lock
          where mbfConnectorTag.PropertyID eq minConnectorTagID.   
    end.
    mchFileMode = DynRetrieve(buffer mbfConnectorTag:handle,'Value').
    ASSIGN
      substring( mchFileMode, 1, 1 ) = (IF iplgHasCreate THEN "C" ELSE " ")
      substring( mchFileMode, 2, 1 ) = (IF iplgHasRead THEN "R" ELSE " ")
      substring( mchFileMode, 3, 1 ) = (IF iplgHasUpdate THEN "U" ELSE " ")
      substring( mchFileMode, 4, 1 ) = (IF iplgHasDelete THEN "D" ELSE " ")
      .
    DynAssign(BUFFER mbfConnectorTag:handle,'Value',mchFileMode).  
    RETURN minConnectorID.
  END METHOD.
  METHOD PUBLIC INTEGER DataLinkCount(input ipinObjectID as INTEGER):
    define variable minCount as integer    no-undo.
    define buffer mbfConnector for Repository.t_Connector.

    for each mbfConnector NO-LOCK
        where (mbfConnector.Start_Object_ID eq ipinObjectID
        OR   mbfConnector.End_Object_ID eq ipinObjectID)
        and   mbfConnector.Connector_Type eq "Dependency"
        AND ReverseMap(mbfConnector.Stereotype) begins "nsDataLink"
        :
        minCount = minCount + 1.
    end.     
    RETURN minCount.
  END METHOD.
  /* manipulacin de diagramas */
  METHOD PUBLIC INTEGER CreateDiagram(input  ipinPackageID as INTEGER,input  ipchName as CHARACTER, INPUT ipchType AS CHARACTER):
    DEFINE BUFFER mbfDiagram FOR Repository.T_Diagram.
    FOR EACH mbfDiagram NO-LOCK
      WHERE mbfDiagram.Package_ID EQ ipinPackageID
      AND mbfDiagram.NAME EQ ipchName ON ERROR UNDO, THROW:
      IF clgIncremental THEN
        return mbfDiagram.Diagram_ID.
      UNDO, THROW NEW AppError("Ya existe un diagrama para el mismo paquete con el mismo nombre",-111).
    END.
  
    IF ipchType EQ ? OR ipchType EQ '' THEN
      ipchType = 'Collaboration'.
    CREATE mbfDiagram.
    ASSIGN
      cinDiagramID = cinDiagramID + 1
      mbfDiagram.Diagram_ID = cinDiagramID
      mbfDiagram.Package_ID = ipinPackageID
      mbfDiagram.NAME = ipchName
      mbfDiagram.Diagram_Type = ipchType
      mbfDiagram.VERSION = '1.0'
      mbfDiagram.Author = "ABL2UML"
      mbfDiagram.EA_GUID = NewEAGUID()
      mbfDiagram.CreatedDate = NOW
      mbfDiagram.ModifiedDate = NOW
      .
    RETURN mbfDiagram.Diagram_ID.
  END METHOD.
  METHOD PUBLIC INTEGER FindDiagram(input  ipinPackageID as INTEGER,input  ipchName as CHARACTER).
    DEFINE BUFFER mbfDiagram FOR Repository.T_Diagram.
    FOR EACH mbfDiagram NO-LOCK
      WHERE mbfDiagram.Package_ID EQ ipinPackageID
      AND mbfDiagram.NAME EQ ipchName:
      RETURN mbfDiagram.Diagram_ID.
    END.
    RETURN ?.
  END METHOD.
  METHOD PUBLIC LOGICAL DeleteDiagram(input  ipinPackageID as INTEGER,input  ipchName as CHARACTER):
    DEFINE BUFFER mbfDiagram FOR Repository.T_Diagram.
    DEFINE BUFFER mbfDiagramObject FOR Repository.T_DiagramObject.
    DEFINE VARIABLE mlgDels AS LOGICAL     NO-UNDO.
    FOR EACH mbfDiagram EXCLUSIVE-LOCK
      WHERE mbfDiagram.Package_ID EQ ipinPackageID
      AND (ipchName EQ ? OR mbfDiagram.NAME EQ ipchName) ON ERROR UNDO, THROW:
      FOR EACH mbfDiagramObject EXCLUSIVE-LOCK
        WHERE mbfDiagramObject.Diagram_ID EQ mbfDiagram.Diagram_ID ON ERROR UNDO, THROW:
        DELETE mbfDiagramObject.
      END.
      DELETE mbfDiagram.
      mlgDels = TRUE.
    END.
    RETURN mlgDels.
  END METHOD.
  METHOD PUBLIC LOGICAL SetDiagramName(input  ipinPackageID as INTEGER,input  ipchName as CHARACTER, INPUT ipchNewName AS CHARACTER):
    DEFINE BUFFER mbfDiagram FOR Repository.T_Diagram.
    FOR EACH mbfDiagram EXCLUSIVE-LOCK
      WHERE mbfDiagram.Package_ID EQ ipinPackageID
      AND mbfDiagram.NAME EQ ipchName ON ERROR UNDO, THROW:
      mbfDiagram.NAME = ipchNewName.
      RETURN TRUE.
    END.
    RETURN FALSE.
  END METHOD.
  METHOD PUBLIC INTEGER MoveDiagram(input  ipinPackageID as INTEGER,input  ipchName as CHARACTER, INPUT ipchNewPackage AS INTEGER):
    DEFINE BUFFER mbfDiagram FOR Repository.T_Diagram.
    FOR EACH mbfDiagram EXCLUSIVE-LOCK
      WHERE mbfDiagram.Package_ID EQ ipinPackageID
      AND mbfDiagram.NAME EQ ipchName ON ERROR UNDO, THROW:
      mbfDiagram.Package_ID = ipchNewPackage.
      RETURN mbfDiagram.Diagram_ID.
    END.
    RETURN ?.
  END METHOD.

  /* oculta todas las relaciones del objeto (para las cuales el objeto es el origen), en el diagrama */
  METHOD PUBLIC VOID HideDiagramRelations(INPUT ipinDiagramID AS INTEGER,INPUT ipinObjectID AS INTEGER):
    DEFINE BUFFER mbfConnector FOR Repository.T_Connector.
    DEFINE BUFFER mbfDiagramLink FOR Repository.T_DiagramLinks.
    DEFINE BUFFER mbfDiagramObject FOR Repository.T_DiagramObjects.
    DEFINE VARIABLE mchEid AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchSid AS CHARACTER   NO-UNDO.

    /* encontrar los DUID de los objetos de inicio y fin del conector, que se necesitan para el campo Style */
    FIND mbfDiagramObject WHERE mbfDiagramObject.Diagram_ID EQ ipinDiagramID AND mbfDiagramObject.OBJECT_ID EQ ipinObjectID.
    mchSid = SUBSTRING(mbfDiagramObject.ObjectStyle,6,8).
    FOR EACH mbfConnector WHERE mbfConnector.START_Object_ID EQ ipinObjectID:
      FIND mbfDiagramLink WHERE mbfDiagramLink.DiagramID EQ ipinDiagramID AND mbfDiagramLink.connectorID EQ mbfConnector.connector_id NO-ERROR.
      IF NOT AVAILABLE mbfDiagramLink THEN DO:
        FIND mbfDiagramObject WHERE mbfDiagramObject.Diagram_ID EQ ipinDiagramID AND mbfDiagramObject.OBJECT_ID EQ mbfConnector.End_Object_ID NO-ERROR.
        IF NOT AVAILABLE mbfDiagramObject THEN /* si no estn los dos extremos del conector, no tiene sentido ocultarlo */
          NEXT.
        mchEid = SUBSTRING(mbfDiagramObject.ObjectStyle,6,8).
        CREATE mbfDiagramLink.
        ASSIGN
          cinDiagramObjectID = cinDiagramObjectID + 1
          mbfDiagramLink.DiagramID = ipinDiagramID
          mbfDiagramLink.ConnectorID = mbfConnector.Connector_ID
          mbfDiagramLink.Instance_ID = cinDiagramObjectID
          mbfDiagramLink.Geometry = 'SX=0;XY=0;EX=0;EY=0;EDGE=1;$LLB=;LLT=;LMT=;LMB=;LRT=;LRB=;IRHS=;ILHS=;'
          mbfDiagramLink.Style = SUBSTITUTE('Mode=3;EOID=&1;SOID=&2;Color=-1;LWidth=0;',mchEid,mchSid)
          .
      END.
      ASSIGN
        mbfDiagramLink.HIDDEN = 1
        .
    END.
  END METHOD.

  METHOD PUBLIC INTEGER AddDiagramObject(input  ipinDiagramID as INTEGER,input  ipinObjectID as INTEGER,INPUT ipchStyle AS CHARACTER,INPUT iplgHideRelations AS LOGICAL):
    DEFINE BUFFER mbfDiagramObject FOR Repository.T_DiagramObject.
    DEFINE VARIABLE mchDUID AS CHARACTER   NO-UNDO.

    FOR EACH mbfDiagramObject NO-LOCK
      WHERE mbfDiagramObject.Diagram_ID EQ ipinDiagramID
      AND mbfDiagramObject.Object_ID EQ ipinObjectID ON ERROR UNDO, THROW:
      IF clgIncremental THEN DO:
        IF iplgHideRelations THEN
          HideDiagramRelations(ipinDiagramID,ipinObjectID).
        RETURN mbfDiagramObject.Instance_ID.
      END.
      UNDO, THROW NEW AppError("Ya existe el objeto para el diagrama",-111).
    END.
  
    DoLog(SUBSTITUTE('Creacin Instancia de Diagrama: &1 - &2',ipinDiagramID,ipinObjectID),'DEBUG').
    /* generar un id nico para el objeto en el diagrama */
    DO WHILE TRUE:
      mchDUID = SUBSTRING(newEAGUID(),30,8).
      IF NOT CAN-FIND(mbfDiagramObject WHERE mbfDiagramObject.Diagram_ID EQ ipinDiagramID AND mbfDiagramObject.objectStyle BEGINS 'DUID=' + mchDUID) THEN
        LEAVE.
    END.
    CREATE mbfDiagramObject.
    ASSIGN
      cinDiagramObjectID = cinDiagramObjectID + 1
      mbfDiagramObject.Instance_ID = cinDiagramObjectID
      mbfDiagramObject.Diagram_ID = ipinDiagramID
      mbfDiagramObject.OBJECT_ID = ipinObjectID
      /* mbfDiagramObject.Sequence = (IF ipchStyle NE '' THEN 1 ELSE 0) */
      mbfDiagramObject.objectstyle = SUBSTITUTE('DUID=&1;&2',mchDUID,ipchStyle)
      .
    /* ocultar del diagrama todas las relaciones de este objeto */
    IF iplgHideRelations THEN
      HideDiagramRelations(ipinDiagramID,ipinObjectID).
    RETURN mbfDiagramObject.Instance_ID.
  END METHOD.
  METHOD PUBLIC LOGICAL ShowDiagramConnector(INPUT ipinDiagramID AS INTEGER, INPUT ipinStartID AS INTEGER, INPUT ipinEndID AS INTEGER, INPUT ipchAlias AS CHARACTER):
    DEFINE BUFFER mbfDiagramLink FOR Repository.T_DiagramLinks.
    DEFINE BUFFER mbfConnector FOR Repository.T_Connector.
    ipchAlias = 'alias=' + ipchAlias + ';'.
    FOR EACH mbfDiagramLink WHERE mbfDiagramLink.DiagramID EQ ipinDiagramID EXCLUSIVE-LOCK: /* ignorar conectores no encontrados */
      FIND mbfConnector WHERE mbfConnector.Connector_ID EQ mbfDiagramLink.ConnectorID.
      IF mbfConnector.START_Object_ID EQ ipinStartID AND mbfConnector.END_Object_ID EQ ipinEndId AND INDEX(mbfConnector.StyleEx,ipchAlias) GT 0 THEN DO:
        mbfDiagramLink.HIDDEN = 0.
        RETURN TRUE.
      END.
    END.
    RETURN FALSE.
  END METHOD.

  METHOD PUBLIC LOGICAL SetPackageName(INPUT ipinPackageId AS INTEGER,INPUT ipchNewName AS CHARACTER):
    define buffer mbfPackage for Repository.t_Package.
    FIND mbfPackage WHERE mbfPackage.Package_ID EQ ipinPackageID EXCLUSIVE-LOCK NO-ERROR.
    IF NOT AVAILABLE mbfPackage THEN
      RETURN FALSE.
    mbfPackage.NAME = ipchNewName.
    RETURN TRUE.
  END METHOD.
  METHOD PUBLIC LOGICAL SetObjectName(INPUT ipinObjectId AS INTEGER,INPUT ipchNewName AS CHARACTER):
    DEFINE BUFFER mbfObject FOR Repository.T_Object.
      
    FIND mbfObject WHERE mbfObject.OBJECT_ID EQ ipinObjectID EXCLUSIVE-LOCK NO-ERROR.
    IF NOT AVAILABLE mbfObject THEN
      RETURN FALSE.
    mbfObject.NAME = ipchNewName.
    FIND cttObject WHERE cttObject.inObjectID EQ ipinObjectID.
    cttObject.chName = ipchNewName.
    RETURN TRUE.
  END METHOD.
  METHOD PUBLIC LOGICAL SetObjectType(INPUT ipinObjectId AS INTEGER,INPUT ipchNewType AS CHARACTER):
    DEFINE BUFFER mbfObject FOR Repository.T_Object.
      
    FIND mbfObject WHERE mbfObject.OBJECT_ID EQ ipinObjectID EXCLUSIVE-LOCK NO-ERROR.
    IF NOT AVAILABLE mbfObject THEN
      RETURN FALSE.
    IF ipchNewType EQ mbfObject.OBJECT_Type THEN
      RETURN TRUE.
    IF ipchNewType EQ 'Package' THEN DO:
      DEFINE BUFFER mbfPackage FOR Repository.T_Package.
      FIND mbfPackage NO-LOCK WHERE mbfPackage.Package_ID EQ mbfObject.Package_ID.
      mbfObject.Package_ID = mbfPackage.Parent_ID.
      mbfObject.PData1 = STRING(mbfPackage.Package_ID).
    END.
    ELSE IF mbfObject.OBJECT_Type EQ 'Package' THEN ASSIGN
      mbfObject.Package_ID = INTEGER(mbfObject.PData1)
      mbfObject.PData1 = ''
      .
    mbfObject.OBJECT_type = ipchNewType.
    FIND cttObject WHERE cttObject.inObjectID EQ ipinObjectID.
    cttObject.chType = ipchNewType.
    RETURN TRUE.
  END METHOD.

  METHOD PRIVATE VOID StartUp(INPUT iplgIncremental AS LOGICAL, INPUT iplgSecure AS LOGICAL):
    /* Check for Repository DB */
    if not connected( "Repository" ) THEN
      undo, THROW NEW AppError("Repository Database must be connected with alias `Repository'",1010 ).
  
    FillStereotypesTT().
    IF iplgIncremental THEN DO:
      FillObjectsTT().
      SetUpSequences().
    END.
    IF iplgSecure THEN DO:
      FOR EACH usys_system EXCLUSIVE-LOCK WHERE NOT usys_system.proper MATCHES '*Ver*':
        DELETE usys_system.
      END.
    END.
    clgIncremental = iplgIncremental.
  END METHOD.

  CONSTRUCTOR EAUtils(INPUT iplgIncremental AS LOGICAL):
    StartUp(iplgIncremental,TRUE).
  END CONSTRUCTOR.

  CONSTRUCTOR EAUtils(INPUT iplgIncremental AS LOGICAL,INPUT iplgSecure AS LOGICAL):
    StartUp(iplgIncremental,iplgSecure).
  END CONSTRUCTOR.

  DESTRUCTOR EAUtils():
    empty temp-table cttObject.
    empty temp-table cttStereotype.
  END DESTRUCTOR.

  /* ILoggeable */
  METHOD PUBLIC ILogger setLogger(INPUT ipobLogger AS ILogger):
    cobLogger = ipobLogger.
    RETURN ipobLogger.
  END METHOD.

  /* limpia una base de datos de repositorio */
  METHOD PROTECTED VOID CleanTable(INPUT iphnBuff AS HANDLE):
    DEFINE VARIABLE mhnQuery AS HANDLE      NO-UNDO.

    CREATE QUERY mhnQuery.
    mhnQuery:SET-BUFFERS(iphnBuff).
    mhnQuery:QUERY-PREPARE('FOR EACH ' + iphnBuff:NAME + ' EXCLUSIVE-LOCK').
    mhnQuery:QUERY-OPEN().
    DO TRANSACTION:
      DO WHILE mhnQuery:GET-NEXT():
        iphnBuff:BUFFER-DELETE.
      END.
    END.
    mhnQuery:QUERY-CLOSE().
    FINALLY:
      DELETE OBJECT mhnQuery.
    END FINALLY.
  END METHOD.

  METHOD PUBLIC VOID CleanIt():
    define buffer mbfAttribute for Repository.t_Attribute.
    define buffer mbfAttributeTag for Repository.t_AttributeTag.
    define buffer mbfConnector for Repository.t_Connector.
    define buffer mbfConnectorTag for Repository.t_ConnectorTag.
    DEFINE BUFFER mbfDiagram FOR Repository.T_Diagram.
    DEFINE BUFFER mbfDiagramLinks FOR Repository.T_DiagramLinks.
    DEFINE BUFFER mbfDiagramObject FOR Repository.T_DiagramObject.
    define buffer mbfObject for Repository.t_Object.
    define buffer mbfObjectProperty for Repository.t_ObjectProperties.
    define buffer mbfOperation for Repository.t_Operation.
    define buffer mbfOperationParams for Repository.t_OperationParams.
    define buffer mbfOperationPost for Repository.t_OperationPosts.
    define buffer mbfOperationTag for Repository.t_OperationTag.
    define buffer mbfPackage for Repository.t_Package.
    define buffer mbfStereotype for Repository.t_Stereotypes.
    define buffer mbfXref for Repository.t_xref.

    CleanTable( BUFFER mbfAttribute:handle). 
    CleanTable( BUFFER mbfAttributeTag:handle).
    CleanTable( BUFFER mbfConnector:handle) .
    CleanTable( BUFFER mbfConnectorTag:handle) .
    CleanTable( BUFFER mbfDiagram:handle) .
    CleanTable( BUFFER mbfDiagramLinks:handle) .
    CleanTable( BUFFER mbfDiagramObject:handle) .
    CleanTable( BUFFER mbfObject:handle).
    CleanTable( BUFFER mbfObjectProperty:handle). 
    CleanTable( BUFFER mbfOperation:handle). 
    CleanTable( BUFFER mbfOperationParams:handle). 
    CleanTable( BUFFER mbfOperationPost:handle). 
    CleanTable( BUFFER mbfOperationTag:handle).
    CleanTable(BUFFER mbfPackage:HANDLE).
    CleanTable( BUFFER mbfStereotype:handle) .
    CleanTable( BUFFER mbfXref:handle).
    empty temp-table cttObject.
    empty temp-table cttStereotype.
    CURRENT-VALUE(CATEGORYID_SEQ) = 0.
    current-value(DATATYPEID_SEQ) = 0.
    current-value(PACKAGE_ID_SEQ) = 0.
    current-value(TRX_ID_SEQ) = 0.
    current-value(GLOSSARYID_SEQ) = 0.
    current-value(OBJECT_ID_SEQ) = 0.
    current-value(DIAGRAM_ID_SEQ) = 0.
    current-value(REQID_SEQ) = 0.
    current-value(OPERATIONID_SEQ) = 0.
    current-value(PALETTEID_SEQ) = 0.
    current-value(ID_SEQ) = 0.
    current-value(CONNECTOR_ID_SEQ) = 0.
    current-value(ISSUEID_SEQ) = 0.
    current-value(IMAGEID_SEQ) = 0.
    current-value(SCRIPTID_SEQ) = 0.
    current-value(TASKID_SEQ) = 0.
    current-value(PATTERNID_SEQ) = 0.
    current-value(PROPERTYID_SEQ) = 0.
    current-value(INSTANCE_ID_SEQ) = 0.
  END METHOD.
END CLASS.
