/*------------------------------------------------------------------------------
    File        : startup.p
    Description : Initial startup procedure for ABL to UML load
    Author(s)   : Thomas Mercer-Hursh
    Created     : Sun Dec 16 12:52:56 PST 2007
    Notes       : Requires:
                  Dictionary database connected with -ld DictSource
                  EA Repository database connected with -ld Repository
                  Dictionary should be connected first, i.e., should be DICTDB
   
  modificaciones 2009/12/15 - D.Abdala
  ----------------------------------------------------------------------------*/
{com/nomadesoft/config.i}
ROUTINE-LEVEL ON ERROR UNDO, THROW.
USING PROGRESS.Lang.AppError.
USING com.nomadesoft.lib.Logger.
USING com.nomadesoft.ABLaUML.UMLBuilder.
USING com.nomadesoft.ABLaUML.DictCleaner.
USING com.nomadesoft.ABLaUML.DiagramCleaner.
&IF {&RUN-PARAMS} &THEN
DEFINE INPUT  PARAMETER ipchParams AS CHARACTER   NO-UNDO.
&ENDIF
/* ***************************  Definitions  ******************************** */
                                                                /* Log */
define variable lobLogger as Logger no-undo.
DEFINE VARIABLE lobBuilder AS CLASS UMLBuilder NO-UNDO.
DEFINE VARIABLE llgIncremental AS LOGICAL   LABEL 'Incremental' INITIAL YES  NO-UNDO.
DEFINE VARIABLE llgBuild AS LOGICAL     NO-UNDO.
DEFINE VARIABLE llgDictFull AS LOGICAL     NO-UNDO.
DEFINE VARIABLE llgAfter AS LOGICAL     NO-UNDO.
DEFINE VARIABLE llgHide AS LOGICAL     NO-UNDO.
DEFINE VARIABLE lchXML AS CHARACTER  LABEL 'Archivo XML' NO-UNDO.
/* ************************* Initialization ********************************* */
                                                             /* Start New Log */
lobLogger = NEW Logger(0,"nsABLaUMLLog.log").
lobLogger:OutLog('Iniciando ABL -> UML',lobLogger:getLevel('MESSAGE')).
                                                        /* Check Repository */
IF not connected( "Repository" )then
  undo, THROW NEW AppError("EA Repository database must be connected with alias `Repository'",2 ).

                                                        /* Determinar el tipo de construccin y crear el constructor */
&IF {&RUN-PARAMS} &THEN
llgIncremental = LOOKUP('incremental',ipchParams) NE 0.
&ELSE
DISPLAY 'Construccin Incremental' WITH FRAME a.
UPDATE llgIncremental WITH 1 COLUMNS FRAME a NO-LABELS.
&ENDIF
lobBuilder = NEW UMLBuilder(lobLogger,llgIncremental).
/* ***************  mapeo de estereotipos  ****************** */
DEFINE VARIABLE lchEMap AS CHARACTER INITIAL 'nsTable,table,nsDataLinkReadOnly,nsDataLinkRead,nsDataLinkModify,nsDataLinkWrite,nsMenuRunLink,nsMenuItemLink,nsMenu,nsMainMenu,nsSubMenu,nsMenu'  NO-UNDO.
&IF {&RUN-PARAMS} &THEN
    lchEMap = ''.
    IF LOOKUP('stereomap',ipchParams) NE 0 THEN
        lchEMap = REPLACE(ENTRY(LOOKUP('stereomap',ipchParams),ipchParams),'|',',').
&ELSE
DISPLAY 'Mapeo de Esterotipos' WITH FRAME a.
UPDATE lchEMap FORMAT "x(280)" VIEW-AS FILL-IN SIZE-CHARS 60 BY 7 WITH 1 COLUMNS FRAME a.
&ENDIF
lobBuilder:cchStereotypeMap = lchEMap.

/* ***************************  Main Block  *************************** */
                                       /* Build UML from data dictionary data */
&IF {&RUN-PARAMS} &THEN
llgBuild = LOOKUP('fullddxml',ipchParams) NE 0.
&ELSE
DISPLAY 'Contruir modelo de datos completo' WITH FRAME b.
UPDATE llgBuild WITH 1 COLUMNS FRAME b NO-LABELS.
&ENDIF
IF llgBuild THEN DO:
  lobLogger:OutLog('Construccin completa del Modelo de Datos',lobLogger:getLevel('MESSAGE')).
  llgDictFull = TRUE.
  &IF {&RUN-PARAMS} &THEN
  lchXML = ENTRY(LOOKUP('fullddxml',ipchParams),ipchParams).
  &ELSE
  UPDATE lchXML FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 WITH 1 COLUMNS FRAME z.
  &ENDIF
  /* agregar al repositorio el modelo de datos completo */
  lobBuilder:Build(lchXML,'com.nomadesoft.ABLaUML.FullDDWorker').
  lobLogger:OutLog('Fin construccin completa del Modelo de Datos',lobLogger:getLevel('MESSAGE')).
END.

                                           /* Build UML from XRef data */
&IF {&RUN-PARAMS} &THEN
llgBuild = LOOKUP('xrefxml',ipchParams) NE 0.
&ELSE
DISPLAY 'Procesar xml XRef' WITH FRAME c.
llgBuild = NO.
UPDATE llgBuild WITH 1 COLUMNS FRAME c NO-LABELS.
&ENDIF
IF llgBuild THEN DO:
    lobLogger:OutLog('Procesando XML XRef',lobLogger:getLevel('MESSAGE')).
    DEFINE VARIABLE llgLink AS LOGICAL NO-UNDO.
    DEFINE VARIABLE llgMove AS LOGICAL NO-UNDO.
    DEFINE VARIABLE llgDiags AS LOGICAL NO-UNDO.
    DEFINE VARIABLE llgCleanIt AS LOGICAL NO-UNDO.
    DEFINE VARIABLE lchWorkers AS CHARACTER   NO-UNDO.
    &IF {&RUN-PARAMS} &THEN
    ASSIGN
      lchXML = ENTRY(LOOKUP('xrefxml',ipchParams),ipchParams)
      llgBuild = LOOKUP('boxref',ipchParams) NE 0
      llgLink = LOOKUP('lkxref',ipchParams) NE 0
      llgMove = LOOKUP('mvxref',ipchParams) NE 0
      llgDiags = LOOKUP('dgxref',ipchParams) NE 0
      llgAfter = LOOKUP('laxref',ipchParams) NE 0
      llgHide = LOOKUP('hdxref',ipchParams) NE 0
      llgCleanIt = LOOKUP('clxref',ipchParams) NE 0
      .
    &ELSE
    UPDATE lchXML FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 WITH 1 COLUMNS FRAME z.
    llgBuild = NO.
    UPDATE llgBuild LABEL 'Crear Objectos' llgLink LABEL 'Crear Enlaces' llgMove LABEL 'Mover UFs' llgDiags LABEL 'Crear Diagramas' llgHide LABEL 'Ocultar Conns.' llgAfter LABEL 'Enlace a Archivo' llgCleanIt LABEL 'Limpiar Tablas en Diags.' WITH 1 COLUMNS FRAME w.
    &ENDIF
    lchWorkers = ''.
    IF llgBuild THEN
      lchWorkers = lchWorkers + ',' + com.nomadesoft.ABLaUML.Classes:ObjectsWorker.
    IF llgLink THEN
      lchWorkers = lchWorkers + ',' + com.nomadesoft.ABLaUML.Classes:LinksWorker.
    IF llgMove THEN
      lchWorkers = lchWorkers + ',' + com.nomadesoft.ABLaUML.Classes:PackageMoveWorker.
    IF llgDiags THEN
      lchWorkers = lchWorkers + ',' + com.nomadesoft.ABLaUML.Classes:DiagramWorker.
    IF llgHide THEN
      lchWorkers = lchWorkers + ',' + com.nomadesoft.ABLaUML.Classes:HideDiagramConnsWorker.
    IF llgAfter THEN
      lchWorkers = lchWorkers + ',com.nomadesoft.ABLaUML.FileLinkWorker'.
    IF lchXML NE '' AND lchWorkers NE '' THEN
      lobBuilder:Build(lchXML,SUBSTRING(lchWorkers,2)).
    lobLogger:OutLog('Fin procesamiento XML XRef',lobLogger:getLevel('MESSAGE')).
    IF llgCleanIt THEN DO ON ERROR UNDO, THROW:
      DEFINE VARIABLE lobDClean AS CLASS DiagramCleaner NO-UNDO.
      lobDClean = NEW DiagramCleaner(lobLogger).
      lobLogger:OutLog('Comienzo limpieza de diagramas',lobLogger:getLevel('MESSAGE')).
      lobDClean:CleanIt().
      lobLogger:OutLog('Fin limpieza de diagramas',lobLogger:getLevel('MESSAGE')).
      FINALLY:
        DELETE OBJECT lobDClean.
      END FINALLY.
    END.
END.

                                        /* Procesar Notas */
&IF {&RUN-PARAMS} &THEN
llgBuild = LOOKUP('notexml',ipchParams) NE 0.
&ELSE
DISPLAY 'Procesar xml Notas' WITH FRAME d.
llgBuild = NO.
UPDATE llgBuild WITH 1 COLUMNS FRAME d NO-LABELS.
&ENDIF
IF llgBuild THEN DO ON ERROR UNDO, THROW:
  lobLogger:OutLog('Procesando XML Notas',lobLogger:getLevel('MESSAGE')).
  &IF {&RUN-PARAMS} &THEN
  lchXML = ENTRY(LOOKUP('notexml',ipchParams),ipchParams).
  llgLink = LOOKUP('lknotas',ipchParams) NE 0.
  &ELSE
  UPDATE lchXML FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 WITH 1 COLUMNS FRAME z.
  UPDATE llgLink LABEL 'Enlace a Archivo' WITH 1 COLUMNS FRAME u.
  &ENDIF
  lchWorkers = 'com.nomadesoft.ABLaUML.NotasWorker'.
  IF llgLink THEN
    lchWorkers = lchWorkers + ',com.nomadesoft.ABLaUML.FileLinkWorker'.
  lobBuilder:Build(lchXML,lchWorkers).
  lobLogger:OutLog('Fin procesamiento XML Notas',lobLogger:getLevel('MESSAGE')).
END.

                                        /* Procesar Alias */
&IF {&RUN-PARAMS} &THEN
llgBuild = LOOKUP('aliasxml',ipchParams) NE 0.
&ELSE
DISPLAY 'Procesar xml Alias' WITH FRAME l.
llgBuild = NO.
UPDATE llgBuild WITH 1 COLUMNS FRAME l NO-LABELS.
&ENDIF
IF llgBuild THEN DO ON ERROR UNDO, THROW:
  lobLogger:OutLog('Procesando XML Alias',lobLogger:getLevel('MESSAGE')).
  &IF {&RUN-PARAMS} &THEN
  lchXML = ENTRY(LOOKUP('aliasxml',ipchParams),ipchParams).
  &ELSE
  UPDATE lchXML FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 WITH 1 COLUMNS FRAME z.
  &ENDIF

  lobBuilder:Build(lchXML,'com.nomadesoft.ABLaUML.AliasWorker').
  lobLogger:OutLog('Fin procesamiento XML Alias',lobLogger:getLevel('MESSAGE')).
END.

                                          /* hacer limpieza del modelo de Datos */
IF llgDictFull THEN DO:
  &IF {&RUN-PARAMS} &THEN
  llgBuild = LOOKUP('ddclean',ipchParams) NE 0.
  &ELSE
  DISPLAY 'Eliminar Modelo de Datos sobrante' WITH FRAME e.
  llgBuild = NO.
  UPDATE llgBuild WITH 1 COLUMNS FRAME e NO-LABELS.
  &ENDIF
  IF llgBuild THEN DO ON ERROR UNDO, THROW:
    DEFINE VARIABLE lobCleaner AS CLASS DictCleaner NO-UNDO.
    lobLogger:OutLog('Limpieza del Modelo de Datos',lobLogger:getLevel('MESSAGE')).
    lobCleaner = NEW DictCleaner(lobLogger).
    lobCleaner:CleanIt().
    lobLogger:OutLog('Fin limpieza del Modelo de Datos',lobLogger:getLevel('MESSAGE')).
    FINALLY:
      DELETE OBJECT lobCleaner.
    END FINALLY.
  END.
END.
                                          /* completar el modelo de datos */
ELSE DO:
  &IF {&RUN-PARAMS} &THEN
  llgBuild = LOOKUP('fillddxml',ipchParams) NE 0.
  &ELSE
  DISPLAY 'Completar Modelo de Datos' WITH FRAME f.
  llgBuild = NO.
  UPDATE llgBuild WITH 1 COLUMNS FRAME f NO-LABELS.
  &ENDIF
  IF llgBuild THEN DO:
    &IF {&RUN-PARAMS} &THEN
    lchXML = ENTRY(LOOKUP('fillddxml',ipchParams),ipchParams).
    &ELSE
    UPDATE lchXML FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 WITH 1 COLUMNS FRAME z.
    &ENDIF
    lobLogger:OutLog('Completando el Modelo de Datos',lobLogger:getLevel('MESSAGE')).
    /* agregar al repositorio el modelo de datos completo */
    lobBuilder:Build(lchXML,'com.nomadesoft.ABLaUML.FillDDWorker').
    lobLogger:OutLog('Fin completado del Modelo de Datos',lobLogger:getLevel('MESSAGE')).
  END.
END.
/* diagramas del modelo de datos */
&IF {&RUN-PARAMS} &THEN
llgBuild = LOOKUP('dddiagram',ipchParams) NE 0.
&ELSE
DISPLAY 'Crear diagramas de datos' WITH FRAME i.
llgBuild = NO.
UPDATE llgBuild WITH 1 COLUMNS FRAME i NO-LABELS.
&ENDIF
IF llgBuild THEN DO:
    lobLogger:OutLog('Creando diagramas del modelo de datos',lobLogger:getLevel('MESSAGE')).
    &IF {&RUN-PARAMS} &THEN
    lchXML = ENTRY(LOOKUP('fillddxml',ipchParams),ipchParams).
    &ELSE
    UPDATE lchXML FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 WITH 1 COLUMNS FRAME z.
    &ENDIF
    lobBuilder:Build(lchXML,'com.nomadesoft.ABLaUML.DiagramDDWorker').
    lobLogger:OutLog('Fin diagramas del modelo de datos',lobLogger:getLevel('MESSAGE')).
END.
/* modelo de menes */
&IF {&RUN-PARAMS} &THEN
llgBuild = LOOKUP('menuxml',ipchParams) NE 0.
&ELSE
DISPLAY 'Procesar xml de menes' WITH FRAME h.
llgBuild = NO.
UPDATE llgBuild WITH 1 COLUMNS FRAME h NO-LABELS.
&ENDIF
IF llgBuild THEN DO:
    DEFINE VARIABLE llgPre AS LOGICAL     NO-UNDO.
    lobLogger:OutLog('Procesando XML de menes',lobLogger:getLevel('MESSAGE')).
    &IF {&RUN-PARAMS} &THEN
    lchXML = ENTRY(LOOKUP('menuxml',ipchParams),ipchParams).
    &ELSE
    UPDATE lchXML FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 WITH 1 COLUMNS FRAME z.
    llgBuild = NO.
    UPDATE llgPre LABEL 'Recrear Primer Nivel' llgBuild LABEL 'Crear Objectos' llgLink LABEL 'Crear Enlaces' llgDiags LABEL 'Crear Diagramas' llgHide LABEL 'Ocultar Conns.' llgMove LABEL  'Renombrar UFs' llgAfter LABEL 'Quitar Primer Nivel' WITH 1 COLUMNS FRAME v.
    &ENDIF
    lchWorkers = ''.
    IF llgPre THEN
      lchWorkers = lchWorkers + ',com.nomadesoft.ABLaUML.MnUndoAfterWorker'.
    IF llgBuild THEN
      lchWorkers = lchWorkers + ',com.nomadesoft.ABLaUML.MnObjectWorker'.
    IF llgLink THEN
      lchWorkers = lchWorkers + ',com.nomadesoft.ABLaUML.MnLinkWorker'.
    IF llgDiags THEN
      lchWorkers = lchWorkers + ',' + com.nomadesoft.ABLaUML.Classes:MnDiagramWorker.
    IF llgHide THEN
      lchWorkers = lchWorkers + ',' + com.nomadesoft.ABLaUML.Classes:MnHideDiagramRelationsWorker.
    IF llgMove THEN
      lchWorkers = lchWorkers + ',com.nomadesoft.ABLaUML.MnFURenameWorker'.
    IF llgAfter THEN
      lchWorkers = lchWorkers + ',com.nomadesoft.ABLaUML.MnAfterWorker'.
    lobBuilder:Build(lchXML,SUBSTRING(lchWorkers,2)).
    lobLogger:OutLog('Fin procesamiento XML de menes',lobLogger:getLevel('MESSAGE')).
END.


CATCH aError AS Progress.Lang.Error :
  DEFINE VARIABLE iinIndex AS INTEGER NO-UNDO.
  lobLogger:OutLog('-- EXCEPCION --',lobLogger:getLevel('ERROR')).
  lobLogger:OutLog(STRING(aError:numMessages) + '~n',lobLogger:getLevel('ERROR')).
  DO iinIndex = 1 TO aError:numMessages :
    lobLogger:OutLog(aError:getMEssage(iinIndex) + '~n',lobLogger:getLevel('ERROR')).
  END.
  IF aError:CallStack NE ? THEN
    lobLogger:OutLog(aError:CallStack,lobLogger:getLevel('ERROR')).
  lobLogger:OutLog('-- EXCEPCION --',lobLogger:getLevel('ERROR')).
  DELETE OBJECT aError.
END CATCH.
/* ****************************  Clean Up  ********************************** */
FINALLY:
  lobLogger:OutLog('Finalizado',lobLogger:getLevel('MESSAGE')).
  DELETE OBJECT lobLogger.
  DELETE OBJECT lobBuilder.
END FINALLY.
/* *************************** End of Program ******************************* */
