/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Genera el xml con la informacin de los programas y sus relaciones, en base
a la informacin de la tabla XREF.
- ENGLISH -
Generates xml with the information of programs and their relations, based
on XREF database content.
*****************/

ROUTINE-LEVEL ON ERROR UNDO, THROW.
USING com.nomadesoft.lib.XMLwriter.


CLASS com.nomadesoft.xml.XRefwriter INHERITS XMLwriter:

  /*
  Quita el punto y la barra del comienzo del nombre 
  */
  METHOD PROTECTED CHARACTER Normalizar(INPUT ipchNombre AS CHARACTER):
    IF ipchNombre EQ ? THEN
      RETURN ?.
    IF SUBSTRING(ipchNombre,1,1) EQ '.' THEN
      ENTRY(1,ipchNombre,'/') = ''.
    IF SUBSTRING(ipchNombre,1,1) EQ '/' THEN
      ipchNombre = SUBSTRING(ipchNombre,2).
    RETURN ipchNombre.
  END METHOD.

  /*
  Determina si un programa corresponde a una FU (es un paquete) o es simplemente
  un programa aislado.
  */
  METHOD PROTECTED LOGICAL esPaquete(INPUT ipchRefFil AS CHARACTER):
    DEFINE BUFFER ibfPaq FOR xrefsrc.xrfprg.

    FIND ibfPaq WHERE xrffil EQ ipchRefFil AND paqprg EQ xrffil NO-ERROR.
    RETURN AVAILABLE ibfPaq.
  END METHOD.

  /* determina si el tipo de acceso a una tabla, desde un .p o .i existe */
  METHOD PROTECTED LOGICAL tieneAccesoDatos (INPUT ipchRefFil AS CHARACTER,INPUT  ipchCU AS CHARACTER,INPUT  ipchObj AS CHARACTER,INPUT  ipchTipo AS CHARACTER):
    DEFINE BUFFER auxRef FOR xrefsrc.xrfdat.
    FIND FIRST auxRef WHERE xrffil EQ ipchRefFil AND srcfil EQ ipchCU AND objidn EQ ipchObj AND reftyp EQ ipchTipo NO-ERROR.

    RETURN AVAILABLE auxRef.
  END METHOD.

  METHOD PROTECTED VOID generarAccesoDatos(INPUT  ipchRefFil AS CHARACTER,INPUT  ipchCU AS CHARACTER):
      DEFINE BUFFER ibfXRef FOR xrefsrc.xrfdat.
      DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.

      FOR EACH ibfXRef WHERE xrffil EQ ipchRefFil AND srcfil EQ ipchCU AND (reftyp EQ 'READ' OR reftyp EQ 'CREATE' OR reftyp EQ 'UPDATE' OR reftyp EQ 'DELETE') BREAK BY objidn ON ERROR UNDO, THROW:
        IF NOT FIRST-OF(objidn) THEN
            NEXT.
        mchAtts = 'table,' + objidn.
        IF tieneAccesoDatos(ipchRefFil,ipchCU,objidn,'READ') THEN
          mchAtts = mchAtts + ',read,true'.
        IF tieneAccesoDatos(ipchRefFil,ipchCU,objidn,'UPDATE') THEN
          mchAtts = mchAtts + ',write,true'.
        IF tieneAccesoDatos(ipchRefFil,ipchCU,objidn,'CREATE') THEN
          mchAtts = mchAtts + ',create,true'.
        IF tieneAccesoDatos(ipchRefFil,ipchCU,objidn,'DELETE') THEN
          mchAtts = mchAtts + ',delete,true'.
        OutputTag('tableref',mchAtts,?).
      END.
  END METHOD.

  /* verifica la cantidad de invocaciones de un archivo, si est invocado en una nica unidad funcional, entonces tiene un padre */
  METHOD PROTECTED CHARACTER UnUnicoPadre(INPUT  ipchXrfFil AS CHARACTER):
    DEFINE BUFFER ibfXRef FOR xrefsrc.xrfdat.
    DEFINE VARIABLE mchPadre AS CHARACTER INITIAL ? NO-UNDO.
    
    /* las inclusiones e invocaciones de programas pueden estar anidados */
    FOR EACH ibfXRef WHERE objidn EQ ipchxrffil BREAK BY xrffil ON ERROR UNDO, THROW:
      IF FIRST-OF(xrffil) THEN DO:
        IF mchPadre NE ? THEN
          RETURN ?.
        mchPadre = xrffil.
      END.
    END.
    RETURN mchPadre.
  END METHOD.

  /* genera el xml de las invocaciones de la fu */
  METHOD PROTECTED VOID generarIncludes(INPUT ipchxrffil AS CHARACTER,INPUT ipchCU AS CHARACTER):
    DEFINE BUFFER ibfXRef FOR xrefsrc.xrfdat.
    DEFINE VARIABLE mchreftyp AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchPaqSrc AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchFu AS CHARACTER   NO-UNDO.

    /* referencias directas a datos */
    generarAccesoDatos(ipchxrffil,ipchCU).
    /* las inclusiones e invocaciones de programas pueden estar anidados */
    FOR EACH ibfXRef WHERE xrffil EQ ipchxrffil AND srcfil EQ ipchCU BREAK BY objidn ON ERROR UNDO, THROW:
      IF NOT FIRST-OF(objidn) THEN
        NEXT.
      IF LOOKUP(reftyp,'RUN,INCLUDE') EQ 0 THEN
        NEXT.
      ASSIGN
        mchFU = ''
        mchPaqSrc = Normalizar(objidn)
        mchreftyp = reftyp
        .
      /* si es una unidad funcional en s misma, entonces debo registrar una llamada, en lugar de un run */
      IF esPaquete(objidn) THEN ASSIGN
        mchFU = ENTRY(1,mchPaqSrc,'.')
        mchreftyp = ?.
      IF mchreftyp EQ 'INCLUDE' THEN
        StartTag('include','file,' + mchPaqSrc).
      ELSE IF mchreftyp EQ 'RUN' THEN
        StartTag('program','file,' + mchPaqSrc).
      ELSE
        StartTag('call','fu,' + mchFU + ',component,' + mchPaqSrc).
      /* evitar las llamadas recursivas!! */
      IF objidn NE ipchCU THEN
        generarIncludes(xrffil,objidn). /* inclusiones realizadas por el include */
      /* si no es un paquete, entonces debo procesar tambin los registros de su compilacin */
      IF mchreftyp EQ 'INCLUDE' OR mchreftyp EQ 'RUN' THEN
        generarIncludes(objidn,objidn).
      EndTag().
    END.
  END METHOD.
  METHOD PUBLIC OVERRIDE VOID writeXML(INPUT ipchDestino AS CHARACTER):
    writeXML('',ipchDestino).
  END METHOD.
  METHOD PUBLIC VOID writeXML(INPUT ipchFilter AS CHARACTER, INPUT ipchDestino AS CHARACTER):
    DEFINE BUFFER ibfPaq FOR xrefsrc.xrfprg.
    DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchPaqSrc AS CHARACTER   NO-UNDO.

    StartXML('bom','generador,NomadeSoft XRef',ipchDestino).

    IF SUBSTRING(ipchFilter,1,1) NE '.' THEN DO:
      IF SUBSTRING(ipchFilter,1,1) NE '/' THEN
        ipchFilter = '/' + ipchFilter.
      ipchFilter = '.' + ipchFilter.
    END.
    IF SUBSTRING(ipchFilter,LENGTH(ipchFilter),1) NE '/' THEN
      ipchFilter = ipchFilter + '/'.

    FOR EACH ibfPaq WHERE paqprg EQ xrffil AND xrffil BEGINS ipchFilter ON ERROR UNDO, THROW:
      mchPaqSrc = Normalizar(paqprg).
      mchPaqSRc = ENTRY(1,mchPaqSRc,'.').
      mchAtts = 'path,' + mchPaqSrc + ','.
      /* verificar si tiene un nico "padre", de ser as especificarlo */
      mchPaqSrc = UnUnicoPadre(paqprg).
      IF mchPaqSrc NE ? THEN
        mchAtts = mchAtts + 'parent,' + ENTRY(1,Normalizar(mchPaqSrc),'.') + ','.
      IF ENTRY(2,xrffil,'.') EQ 'i' THEN
        mchAtts = mchAtts + 'include,'.
      ELSE
        mchAtts = mchAtts + 'program,'.
      StartTag('fu',mchAtts  + Normalizar(xrffil)).
      /* agregar el programa inicial */
      generarIncludes(xrffil,xrffil).
      EndTag().
    END.
    FinishXML().
  END METHOD.
END CLASS.
